# -*- coding: utf-8 -*-
# Copyright (C) 2017-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django import forms
from django.utils.translation import gettext_lazy as _


class MemberForm(forms.Form):
    """Assign a role to the member"""

    email = forms.EmailField(
        label=_('Email Address'),
        error_messages={
            'required': _('Please enter an email address.'),
            'invalid': _('Please enter a valid email address.'),
        },
    )
    display_name = forms.CharField(
        label=_('Display Name'),
        required=False,
        error_messages={'invalid': _('Please enter a display name.')},
    )



# -*- coding: utf-8 -*-
# Copyright (C) 2020-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django import forms
from django.utils.translation import gettext_lazy as _


class AddBanForm(forms.Form):
    """Ban an email address or regular expression."""

    # TODO maxking: This form should only accept valid emails or regular
    # expressions. Anything else that doesn't look like a valid email address
    # or regexp for email should not be a valid value for the field. However,
    # checking for that might not be easy.
    email = forms.CharField(
        label=_('Add ban'),
        help_text=_(
            'You can ban a single email address or use a regular expression '
            'to match similar email addresses.'
        ),
        error_messages={
            'required': _('Please enter an email address.'),
            'invalid': _('Please enter a valid email address.'),
        },
    )



# Generated by Django 2.0.6 on 2018-07-11 20:59

from django.db import migrations, models


# flake8: noqa


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0005_auto_20180707_1107'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='data',
            field=models.TextField(help_text="Note: Do not add any secret content in templates as they are publicly accessible.\nYou can use these variables in the templates. \n$hyperkitty_url: Permalink to archived message in Hyperkitty\n$listname: Name of the Mailing List e.g. ant@example.com \n$list_id: The List-ID header e.g. ant.example.com \n$display_name: Display name of the mailing list e.g. Ant \n$short_listname: Local part of the listname e.g. ant \n$domain: The domain part of the listname e.g. example.com \n$info: The mailing list's longer descriptive text \n$request_email: The email address for -request address \n$owner_email: The email address for -owner address \n$site_email: The email address to reach the owners of the site \n$language: The two letter language code for list's preferred language e.g. fr, en, de \n"),
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

from django.core.management.base import BaseCommand

from django_mailman3.lib.mailman import get_mailman_client


class Command(BaseCommand):
    help = """Opens a Python shell with a mailmanclient object named `client`.

Usage example:
    client.lists
    [<List "foo@example.org">]
    foo = client.get_list('foo@example.org')
    foo.members
    [<Member "les@primus.org">]

A complete list of commands can be found in the mailmanclient documentation."""

    def handle(self, *args, **options):
        # choose an interpreter
        try:
            import IPython

            console_fn = IPython.embed
        except ImportError:
            import code

            shell = code.InteractiveConsole(globals())
            console_fn = shell.interact
        # connect to mailmanclient
        client = get_mailman_client()
        # Putting client back in the global scope
        globals()['client'] = client
        # run the interpreter
        console_fn()



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django import template
from django.utils.translation import gettext_lazy as _


register = template.Library()

translation_msgids = {
    'msgid:title:list_header_matches': _('Header Filters'),
    'msgid:title:list_bans': _('Banned Addresses'),
    'msgid:title:list_delete': _('Delete List'),
    'msgid:title:list_held_messages': _('Held Messages'),
    'msgid:title:list_mass_removal': _('Mass Removal'),
    'msgid:title:list_mass_subscription': _('Mass Subscription'),
    'msgid:title:list_members': _('Subscription Options'),
    'msgid:title:list_settings': _('List Settings'),
    'msgid:title:list_summary': _('Summary'),
    'msgid:title:list_mass_removal_confirm': _(
        'Confirm Removal of All Members'
    ),
    'msgid:title:list_templates': _('Templates'),
    'msgid:title:user_settings_address': _('Address-based Settings'),
    'msgid:title:user_settings_list': _('Subscription Settings'),
    'msgid:title:user_settings_global': _('Global Settings'),
    'msgid:title:user_subscriptions': _('Subscriptions'),
}


@register.inclusion_tag('postorius/menu/list_nav.html', takes_context=True)
def list_nav(context, current, title='', subtitle=''):
    title = translation_msgids.get(title, title)
    subtitle = translation_msgids.get(subtitle, subtitle)
    return dict(
        list=context['list'],
        current=current,
        user=context['request'].user,
        title=title,
        subtitle=subtitle,
    )


@register.inclusion_tag('postorius/menu/user_nav.html', takes_context=True)
def user_nav(context, current, title='', subtitle=''):
    title = translation_msgids.get(title, title)
    subtitle = translation_msgids.get(subtitle, subtitle)
    return dict(
        current=current,
        user=context['request'].user,
        title=title,
        subtitle=subtitle,
    )


@register.simple_tag(takes_context=True)
def nav_active_class(context, current, view_name):
    if current == view_name:
        return 'active'
    return ''


@register.filter
def held_count(mlist):
    return mlist.get_held_count()


@register.filter
def pending_subscriptions(mlist):
    return mlist.get_requests_count(token_owner='moderator')



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-07-02 04:44
from __future__ import unicode_literals

from django.db import migrations, models


# flake8: noqa

class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('postorius', '0003_drop_addressconfirmationprofile'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(choices=[('list:admin:action:post', 'Sent to the list administrators when moderator approval for a posting is required.'), ('list:admin:action:subscribe', 'Sent to the list administrators when moderator approval for a subscription request is required.'), ('list:admin:action:unsubscribe', 'Sent to the list administrators when moderator approval for an unsubscription request is required.'), ('list:admin:notice:subscribe', 'Sent to the list administrators to notify them when a new member has been subscribed.'), ('list:admin:notice:unrecognized', 'Sent to the list administrators when a bounce message in an unrecognized format has been received.'), ('list:admin:notice:unsubscribe', 'Sent to the list administrators to notify them when a member has been unsubscribed.'), ('list:member:digest:footer', 'The footer for a digest message.'), ('list:member:digest:header', 'The header for a digest message.'), ('list:member:digest:masthead', 'The digest “masthead”; i.e. a common introduction for all digest messages.'), ('list:member:regular:footer', 'The footer for a regular (non-digest) message.'), ('list:member:regular:header', 'The header for a regular (non-digest) message.'), ('list:user:action:subscribe', 'The message sent to subscribers when a subscription confirmation is required.'), ('list:user:action:unsubscribe', 'The message sent to subscribers when an unsubscription confirmation is required.'), ('list:user:notice:goodbye', 'The notice sent to a member when they unsubscribe from a mailing list.'), ('list:user:notice:hold', 'The notice sent to a poster when their message is being held or moderator approval.'), ('list:user:notice:no-more-today', 'Sent to a user when the maximum number of autoresponses has been reached for that day.'), ('list:user:notice:post', 'Notice sent to a poster when their message has been received by the mailing list.'), ('list:user:notice:probe', 'A bounce probe sent to a member when their subscription has been disabled due to bounces.'), ('list:user:notice:refuse', 'Notice sent to a poster when their message has been rejected by the list’s moderator.'), ('list:user:notice:welcome', 'The notice sent to a member when they are subscribed to the mailing list.')], help_text='Choose the template you want to customize.', max_length=100)),
                ('data', models.TextField(help_text="You can use these variables in the templates. \n$hyperkitty_url: Permalink to archived message in Hyperkitty\n$listname: Name of the Mailing List e.g. ant@example.com \n$list_id: The List-ID header e.g. ant.example.com \n$display_name: Display name of the mailing list e.g. Ant \n$short_listname: Local part of the listname e.g. ant \n$domain: The domain part of the listname e.g. example.com \n$info: The mailing list's longer descriptive text \n$request_email: The email address for -request address \n$owner_email: The email address for -owner address \n$site_email: The email address to reach the owners of the site \n$language: The two letter language code for list's preferred language e.g. fr, en, de \n")),
                ('language', models.CharField(blank=True, choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokmål'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese')], help_text="Language for the template, this should be the list's preferred language.", max_length=5)),
                ('craeted_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('context', models.CharField(choices=[('site', 'Site Wide'), ('domain', 'Domain Wide'), ('list', 'MailingList Wide')], max_length=50)),
                ('identifier', models.CharField(blank=True, max_length=100)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='emailtemplate',
            unique_together=set([('name', 'identifier', 'language')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

"""
Django settings for postorius project.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.9/ref/settings/
"""

# Compatibility with Bootstrap 3
from django.contrib.messages import constants as messages


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '$!-7^wl#wiifjbh)5@f7ji%x!vp7s1vzbvwt26hxv$idixq0u0'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ADMINS = (
    # ('Admin', 'webmaster@example.com'),
)

SITE_ID = 1

ALLOWED_HOSTS = []

# Mailman API credentials
MAILMAN_REST_API_URL = 'http://localhost:8001'
MAILMAN_REST_API_USER = 'restadmin'
MAILMAN_REST_API_PASS = 'restpass'


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'postorius',
    'django_mailman3',
    'django_gravatar',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'allauth.socialaccount.providers.openid',
    'django_mailman3.lib.auth.fedora',
    'allauth.socialaccount.providers.github',
    'allauth.socialaccount.providers.gitlab',
    'allauth.socialaccount.providers.google',
    # 'allauth.socialaccount.providers.facebook',
    'allauth.socialaccount.providers.twitter',
    'allauth.socialaccount.providers.stackexchange',
    # Dev only dependencies. Do not include in any production site.
    # 'debug_toolbar',
    # 'requests_panel',
)


MIDDLEWARE = [
    'allauth.account.middleware.AccountMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    # NOTE: Do not include DebugToolbarMiddleware in any production site.
    # 'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django_mailman3.middleware.TimezoneMiddleware',
    'postorius.middleware.PostoriusMiddleware',
]

# Set `postorius.urls` as main url config if Postorius
# is the only app you want to serve.
ROOT_URLCONF = 'urls'


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django_mailman3.context_processors.common',
                'postorius.context_processors.postorius',
            ],
        },
    },
]

WSGI_APPLICATION = 'wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': '/var/lib/postorius/data/postorius.db',
    }
}

# Maintain type of autogenerated keys going forward
# https://docs.djangoproject.com/en/3.2/releases/3.2/#customizing-type-of-auto-created-primary-keys
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

# Password validation
# https://docs.djangoproject.com/en/1.9/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/


# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = '/var/lib/postorius/static'

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/static/'

LOGIN_URL = 'account_login'
LOGIN_REDIRECT_URL = 'list_index'
LOGOUT_URL = 'account_logout'


# From Address for emails sent to users
DEFAULT_FROM_EMAIL = 'postorius@localhost.local'
# From Address for emails sent to admins
SERVER_EMAIL = 'root@localhost.local'
MESSAGE_TAGS = {messages.ERROR: 'danger'}


AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'allauth.account.auth_backends.AuthenticationBackend',
)

# Django Allauth
ACCOUNT_AUTHENTICATION_METHOD = 'username_email'
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_EMAIL_VERIFICATION = 'mandatory'
ACCOUNT_DEFAULT_HTTP_PROTOCOL = 'https'
ACCOUNT_UNIQUE_EMAIL = True

SOCIALACCOUNT_PROVIDERS = {
    'openid': {
        'SERVERS': [
            dict(id='yahoo', name='Yahoo', openid_url='http://me.yahoo.com'),
        ],
    },
    'google': {
        'SCOPE': ['profile', 'email'],
        'AUTH_PARAMS': {'access_type': 'online'},
    },
    'facebook': {
        'METHOD': 'oauth2',
        'SCOPE': ['email'],
        'FIELDS': [
            'email',
            'name',
            'first_name',
            'last_name',
            'locale',
            'timezone',
        ],
        'VERSION': 'v2.4',
    },
}


# These can be set to override the defaults but are not mandatory:
# EMAIL_CONFIRMATION_TEMPLATE = 'postorius/address_confirmation_message.txt'
# EMAIL_CONFIRMATION_SUBJECT = 'Confirmation needed'


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'simple',
        },
        'file': {
            'level': 'INFO',
            #'class': 'logging.handlers.RotatingFileHandler',
            'class': 'logging.handlers.WatchedFileHandler',
            'filename': '/var/lib/postorius/data/postorius.log',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console', 'file'],
            'level': 'INFO',
        },
        'django.request': {
            'handlers': ['console', 'file'],
            'level': 'ERROR',
        },
        'postorius': {
            'handlers': ['console', 'file'],
            'level': 'DEBUG',
        },
    },
    'formatters': {
        'simple': {'format': '%(levelname)s: %(message)s'},
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(process)d %(name)s %(message)s'
        },
    },
}


POSTORIUS_TEMPLATE_BASE_URL = 'http://localhost:8000'

#: Debug the API calls made by Postorius.
POSTORIUS_API_DEBUG = False

if DEBUG and POSTORIUS_API_DEBUG:
    MIDDLEWARE.append('postorius.middleware.APICountingMiddleware')


# Needed for django debug toolbar to show up.
INTERNAL_IPS = ('127.0.0.1',)

# List of panels that show up for django-debug-toolbar. They are
# very useful for debugging and are ONLY REQUIRED FOR TESTING.
DEBUG_TOOLBAR_PANELS = [
    'requests_panel.panel.RequestsDebugPanel',
    'debug_toolbar.panels.sql.SQLPanel',
    'debug_toolbar.panels.templates.TemplatesPanel',
    'debug_toolbar.panels.history.HistoryPanel',
    'debug_toolbar.panels.versions.VersionsPanel',
    'debug_toolbar.panels.timer.TimerPanel',
    'debug_toolbar.panels.settings.SettingsPanel',
    'debug_toolbar.panels.headers.HeadersPanel',
    'debug_toolbar.panels.request.RequestPanel',
    'debug_toolbar.panels.staticfiles.StaticFilesPanel',
    'debug_toolbar.panels.cache.CachePanel',
    'debug_toolbar.panels.signals.SignalsPanel',
    'debug_toolbar.panels.logging.LoggingPanel',
    'debug_toolbar.panels.redirects.RedirectsPanel',
    'debug_toolbar.panels.profiling.ProfilingPanel',
]

try:
    from settings_local import *
except ImportError:
    pass



# Generated by Django 3.2.12 on 2022-03-04 13:12

from django.db import migrations, models


# flake8: noqa

class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0017_alter_emailtemplate_language'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='language',
            field=models.CharField(blank=True, choices=[('ar', 'Arabic'), ('ast', 'Asturian'), ('bg', 'Bulgarian'), ('ca', 'Catalan'), ('zh_CN', 'Chinese'), ('zh_TW', 'Chinese (Taiwan)'), ('hr', 'Croatian'), ('cs', 'Czech'), ('da', 'Danish'), ('nl', 'Dutch'), ('en', 'English (USA)'), ('et', 'Estonian'), ('eu', 'Euskara'), ('fi', 'Finnish'), ('fr', 'French'), ('gl', 'Galician'), ('de', 'German'), ('el', 'Greek'), ('he', 'Hebrew'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('it', 'Italian'), ('ja', 'Japanese'), ('ko', 'Korean'), ('lt', 'Lithuanian'), ('no', 'Norwegian'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt_BR', 'Portuguese (Brazil)'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sr', 'Serbian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('es', 'Spanish'), ('sv', 'Swedish'), ('tr', 'Turkish'), ('uk', 'Ukrainian'), ('vi', 'Vietnamese')], help_text="Language for the template, this should be the list's preferred language.", max_length=5),
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


import csv
import email.utils
import logging
import sys
from urllib.error import HTTPError

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.forms import formset_factory
from django.http import Http404, HttpResponse
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy
from django.views.decorators.http import require_http_methods

from allauth.account.models import EmailAddress
from django_mailman3.lib.mailman import (
    get_mailman_client,
    get_mailman_user,
    get_mailman_user_id,
)
from django_mailman3.lib.paginator import MailmanPaginator, paginate
from django_mailman3.models import MailDomain
from django_mailman3.signals import (
    mailinglist_created,
    mailinglist_deleted,
    mailinglist_modified,
)

from postorius.auth.decorators import (
    list_moderator_required,
    list_owner_required,
    superuser_required,
)
from postorius.auth.mixins import UserShowListMembersMixin
from postorius.forms import (
    AlterMessagesForm,
    ArchiveSettingsForm,
    BounceProcessingForm,
    ChangeSubscriptionForm,
    DigestSettingsForm,
    DMARCMitigationsForm,
    ListAnonymousSubscribe,
    ListAutomaticResponsesForm,
    ListHeaderMatchForm,
    ListHeaderMatchFormset,
    ListIdentityForm,
    ListMassRemoval,
    ListMassSubscription,
    ListNew,
    ListSubscribe,
    MemberForm,
    MemberModeration,
    MemberPolicyForm,
    MessageAcceptanceForm,
    MultipleChoiceForm,
    UserPreferences,
)
from postorius.forms.fields import (
    DELIVERY_MODE_CHOICES,
    DELIVERY_STATUS_CHOICES,
)
from postorius.forms.list_forms import ACTION_CHOICES, TokenConfirmForm
from postorius.models import (
    Domain,
    List,
    Mailman404Error,
    Style,
    SubscriptionMode,
)
from postorius.utils import get_member_or_nonmember, set_preferred
from postorius.views.generic import MailingListView, bans_view


logger = logging.getLogger(__name__)


#: DeliveryStatus field values that an admin cannot set.
DISABLED_DELIVERY_STATUS_CHOICES_ADMIN = ['by_bounces']

DELIVERY_MODE_DICT = dict(DELIVERY_MODE_CHOICES)
DELIVERY_STATUS_DICT = dict(DELIVERY_STATUS_CHOICES)


class TokenOwner:
    """Who 'owns' the token returned from the registrar?"""

    subscriber = 'subscriber'
    moderator = 'moderator'


class ListMembersViews(UserShowListMembersMixin, MailingListView):

    # List of allowed roles for the memberships. The string value matches the
    # exact value Core's REST API expects.
    allowed_roles = ['owner', 'moderator', 'member', 'nonmember']

    def _prepare_query(self, request):
        """Prepare regex based query to search partial email addresses.

        Core's `members/find` API allows searching for memberships based on
        regex. This methods prepares a valid regex to pass on the REST API.
        """
        if request.GET.get('q'):
            query = request.GET['q']
            if '*' not in query:
                query = '*{}*'.format(query)
        else:
            query = ''

        return query

    def get(self, request, list_id, role):
        """Handle GET for Member view.

        This includes all the membership roles (self.allowed_roles).
        """
        member_form = MemberForm()
        # If the role is misspelled, redirect to the default subscribers.
        if role not in self.allowed_roles:
            return redirect('list_members', list_id, 'member')
        context = dict()
        context['list'] = self.mailing_list
        # Warning: role not translatable
        context['role'] = role
        context['member_form'] = member_form
        context['page_title'] = _('List {}s'.format(role.capitalize()))
        context['query'] = self._prepare_query(request)

        def find_method(count, page):
            return self.mailing_list.find_members(
                context['query'], role=role, count=count, page=page
            )

        context['members'] = paginate(
            find_method,
            request.GET.get('page', 1),
            request.GET.get('count', 25),
            paginator_class=MailmanPaginator,
        )
        context['page_subtitle'] = '({})'.format(
            context['members'].object_list.total_size
        )
        context['form_action'] = _('Add {}'.format(role))
        if context['query']:
            context['empty_error'] = _(
                'No {}s were found matching the search.'.format(role)
            )
        else:
            context['empty_error'] = _('List has no {}s'.format(role))

        return render(request, 'postorius/lists/members.html', context)

    def _member_post(self, request, role):
        """Handle POST for members. Unsubscribe all the members selected."""
        form = MultipleChoiceForm(request.POST)
        if form.is_valid():
            members = form.cleaned_data['choices']
            for member in members:
                self.mailing_list.unsubscribe(member)
            messages.success(
                request, _('The selected members have been unsubscribed')
            )
        return redirect('list_members', self.mailing_list.list_id, role)

    def _non_member_post(self, request, role):
        """Handle POST for membership roles owner, moderator and non-member.

        Add memberships if the form is valid otherwise redirect to list_members
        page with an error message.
        """
        member_form = MemberForm(request.POST)
        if member_form.is_valid():
            try:
                self.mailing_list.add_role(
                    role=role,
                    address=member_form.cleaned_data['email'],
                    display_name=member_form.cleaned_data['display_name'],
                )
                messages.success(
                    request,
                    _(
                        '{email} has been added with the role {role}'.format(
                            email=member_form.cleaned_data['email'], role=role
                        )
                    ),
                )
            except HTTPError as e:
                messages.error(request, e.msg)
        else:
            messages.error(request, member_form.errors)
        return redirect('list_members', self.mailing_list.list_id, role)

    def post(self, request, list_id, role=None):
        """Handle POST for list members page.

        List members page have more than one forms, depending on the membership
        type.

        - Regular subscribers have a MultipleChoiceForm, which returns a list
          of emails that needs to be unsubscribed from the MailingList. This is
          handled by :func:`self._member_post` method.

        - Owners, moderators and non-members have a MemberForm which allows
          adding new members with the given roles. This is handled by
          :func:`self._non_member_post` method.

        """
        if role not in self.allowed_roles:
            return redirect('list_members', list_id, 'member')

        if role in ('member',):
            return self._member_post(request, role)
        else:
            return self._non_member_post(request, role)


@login_required
@list_owner_required
def list_member_options(request, list_id, email):
    template_name = 'postorius/lists/memberoptions.html'
    mm_list = List.objects.get_or_404(fqdn_listname=list_id)
    # If the role is specified explicitly, use that, otherwise the value is
    # None and is equivalent to the value not being specified and the lookup
    # happens using only email. This can cause issues when the email is
    # subscribed as both a Non-Member and Owner/Moderator returning the wrong
    # Member object.
    role = request.GET.get('role')
    try:
        mm_member = mm_list.find_members(address=email, role=role)[0]
        member_prefs = mm_member.preferences
    except (ValueError, IndexError):
        raise Http404(_('Member does not exist'))
    except Mailman404Error:
        return render(request, template_name, {'nolists': 'true'})

    initial_moderation = dict(
        [
            (key, getattr(mm_member, key))
            for key in MemberModeration.base_fields
        ]
    )

    if request.method == 'POST':
        if request.POST.get('formname') == 'preferences':
            preferences_form = UserPreferences(
                request.POST,
                preferences=member_prefs,
                disabled_delivery_choices=DISABLED_DELIVERY_STATUS_CHOICES_ADMIN,  # noqa: E501
            )
            if preferences_form.is_valid():
                try:
                    preferences_form.save()
                except HTTPError as e:
                    messages.error(request, e.msg)
                else:
                    messages.success(
                        request,
                        _("The member's preferences" ' have been updated.'),
                    )
            return redirect('list_member_options', list_id, email)
        elif request.POST.get('formname') == 'moderation':
            moderation_form = MemberModeration(
                request.POST, initial=initial_moderation
            )
            if moderation_form.is_valid():
                if not moderation_form.has_changed():
                    messages.info(
                        request, _("No change to the member's moderation.")
                    )
                    return redirect('list_member_options', list_id, email)
                for key in list(moderation_form.fields.keys()):
                    # In general, it would be a very bad idea to loop over the
                    # fields and try to set them one by one, However,
                    # moderation form has only one field.
                    setattr(mm_member, key, moderation_form.cleaned_data[key])
                try:
                    mm_member.save()
                except HTTPError as e:
                    messages.error(request, e.msg)
                else:
                    messages.success(
                        request,
                        _(
                            "The member's moderation "
                            'settings have been updated.'
                        ),
                    )
                return redirect('list_member_options', list_id, email)
    else:
        preferences_form = UserPreferences(
            preferences=member_prefs,
            disabled_delivery_choices=DISABLED_DELIVERY_STATUS_CHOICES_ADMIN,
        )
        moderation_form = MemberModeration(initial=initial_moderation)
    return render(
        request,
        template_name,
        {
            'mm_member': mm_member,
            'list': mm_list,
            'preferences_form': preferences_form,
            'moderation_form': moderation_form,
        },
    )


class ListSummaryView(MailingListView):
    """Shows common list metrics."""

    def get(self, request, list_id):
        data = {
            'list': self.mailing_list,
            'user_subscribed': False,
            'subscribed_address': None,
            'subscribed_preferred': False,
            'public_archive': False,
            'hyperkitty_enabled': False,
            'anon_subscribe': getattr(
                settings, 'SHOW_ANONYMOUS_SUBSCRIBE_FORM', True
            ),
        }
        if self.mailing_list.settings['archive_policy'] == 'public':
            data['public_archive'] = True
        if (
            getattr(settings, 'TESTING', False)
            and 'hyperkitty' not in settings.INSTALLED_APPS  # noqa: W504
        ):
            # avoid systematic test failure when HyperKitty is installed
            # (missing VCR request, see below).
            list(self.mailing_list.archivers)
        archivers = self.mailing_list.archivers
        if (
            'hyperkitty' in settings.INSTALLED_APPS
            and 'hyperkitty' in archivers  # noqa: W504
            and archivers['hyperkitty']  # noqa: W504
        ):                       # noqa: W504
            data['hyperkitty_enabled'] = True
        if request.user.is_authenticated:
            user_emails = (
                EmailAddress.objects.filter(user=request.user, verified=True)
                .order_by('email')
                .values_list('email', flat=True)
            )
            pending_requests = [r['email'] for r in self.mailing_list.requests]

            subscriptions = []
            for address in user_emails:
                if address in pending_requests:
                    data['user_request_pending'] = True
                    continue
                try:
                    member = self.mailing_list.get_member(address)
                except ValueError:
                    pass
                else:
                    subscriptions.append(
                        {
                            'subscribed_address': address,
                            'subscriber': member.member_id,
                            'subscribed_preferred': bool(
                                member.subscription_mode
                                == SubscriptionMode.as_user.name
                            ),
                            'delivery_mode': DELIVERY_MODE_DICT.get(
                                member.preferences.get('delivery_mode')
                            ),
                            'delivery_status': DELIVERY_STATUS_DICT.get(
                                member.preferences.get('delivery_status')
                            ),
                        }
                    )
            data['user_subscriptions'] = subscriptions
            data['user_subscribed'] = bool(subscriptions)

            mm_user = get_mailman_user(request.user)
            primary_email = None
            if mm_user.preferred_address is None:
                primary_email = set_preferred(request.user, mm_user)
            else:
                primary_email = mm_user.preferred_address.email
            data['subscribe_form'] = ListSubscribe(
                user_emails,
                user_id=mm_user.user_id,
                primary_email=primary_email,
            )
        else:
            user_emails = None
            data['anonymous_subscription_form'] = ListAnonymousSubscribe()
        return render(request, 'postorius/lists/summary.html', data)


class ChangeSubscriptionView(MailingListView):
    """Change mailing list subscription"""

    @staticmethod
    def _is_email(subscriber):
        """Is the subscriber an email address or uuid."""
        return '@' in subscriber

    def _change_subscription(self, member, subscriber):
        """Switch subscriptions to a new email/user.

        :param member: The currently subscriber Member.
        :param subscriber: New email or user_id to switch subscription to.
        """
        # If the Membership is via preferred address and we are switching to an
        # email or vice versa, we have to do the subscribe-unsubscribe dance.
        # If the subscription is via an address, then we can simply PATCH the
        # address in the Member resource.
        if (
            member.subscription_mode == SubscriptionMode.as_address.name
            and self._is_email(subscriber)
        ):
            member.address = subscriber
            member.save()
            return
        # Keep a copy of the preferences so we can restore them after the
        # Member switches addresses.
        if member.preferences:
            prefs = member.preferences
        member.unsubscribe()
        # Unless the subscriber is an an email and it is not-verified, we
        # should get a Member resource here as response. We should never get to
        # here with subscriber being an un-verified email.
        new_member = self.mailing_list.subscribe(
            subscriber,
            # Since the email is already verified in Postorius.
            pre_confirmed=True,
            # Since this user was already a Member, simply switching Email
            # addresses shouldn't require another approval.
            pre_approved=True,
        )
        self._copy_preferences(new_member.preferences, prefs)

    def _copy_preferences(self, member_pref, old_member_pref):
        """Copy the preferences of the old member to new one.

        :param member_pref: The new member's preference to copy preference to.
        :param old_member_pref: The old_member's preferences to copy
            preferences from.
        """
        # We can't simply switch the Preferences object, so we copy values from
        # previous one to the new one.
        for prop in old_member_pref._properties:
            val = old_member_pref.get(prop)
            if val:
                member_pref[prop] = val
        member_pref.save()

    def _get_membership(self, mm_user, member_id):
        """Get current memberships of the Mailman user."""
        for sub in mm_user.subscriptions:
            if sub.member_id == member_id:
                return sub
        return None

    def _is_subscribed(self, member, subscriber, mm_user):
        """Check if the member is same as the subscriber.

        If they are not:
        - If subscriber is 'user_id' check if member's user_id is same as
           subscriber.
        - if subscriber is address, check if member's address is same as
           subscriber.
        """
        return (
            member.subscription_mode == SubscriptionMode.as_address.name
            and member.email == subscriber
        ) or (
            member.subscription_mode == SubscriptionMode.as_user.name
            and member.user.user_id == subscriber
        )

    @method_decorator(login_required)
    def post(self, request, list_id):
        try:
            user_emails = (
                EmailAddress.objects.filter(user=request.user, verified=True)
                .order_by('email')
                .values_list('email', flat=True)
            )
            mm_user = get_mailman_user(request.user)
            primary_email = None
            if mm_user.preferred_address is not None:
                primary_email = mm_user.preferred_address.email
            form = ChangeSubscriptionForm(
                user_emails, mm_user.user_id, primary_email, request.POST
            )

            if form.is_valid():
                # Step 1: Get the source membership user wants to switch from.
                member = self._get_membership(
                    mm_user, form.cleaned_data['member_id']
                )
                if member is None:
                    # Source membership doesn't exist, use should use subscribe
                    # form instead.
                    messages.error(
                        _('You are not subscribed to {}.').format(
                            self.mailing_list.fqdn_listname
                        )
                    )
                    # Not sure what the right place to return to here, since
                    # there is no on_get resource here. Return to
                    # per-subscription-preferences where the user can choose
                    # subscription to change.
                    return redirect('user_subscription_preferences')
                subscriber = form.cleaned_data['subscriber']
                # Step 2: Check if the source and destination subscriber are
                # the same. This means, if the subscriber is a user_id, check
                # they match and if the subscriber is a email address, check if
                # they match.
                if self._is_subscribed(member, subscriber, mm_user):
                    messages.error(request, _('You are already subscribed'))
                else:
                    self._change_subscription(member, subscriber)
                    # If the subscriber is user_id (i.e. not an email
                    # address), Show 'Primary Address ()' in the success
                    # message instead of weird user id.
                    if not self._is_email(subscriber):
                        subscriber = _('Primary Address ({})').format(
                            primary_email
                        )
                    messages.success(
                        request,
                        _('Subscription changed to {}').format(subscriber),
                    )
            else:
                messages.error(
                    request,
                    _('Something went wrong. Please try again. {} {}').format(
                        form.errors, form.fields['subscriber'].choices
                    ),
                )
        except HTTPError as e:
            messages.error(request, e.msg)
        return redirect('list_summary', self.mailing_list.list_id)


class ListSubscribeView(MailingListView):
    """
    view name: `list_subscribe`
    """

    @method_decorator(login_required)
    def post(self, request, list_id):
        """
        Subscribes an email address to a mailing list via POST and
        redirects to the `list_summary` view.
        """
        try:
            user_emails = (
                EmailAddress.objects.filter(user=request.user, verified=True)
                .order_by('email')
                .values_list('email', flat=True)
            )
            mm_user = get_mailman_user(request.user)
            primary_email = None
            if mm_user.preferred_address is not None:
                primary_email = mm_user.preferred_address.email
            form = ListSubscribe(
                user_emails, mm_user.user_id, primary_email, request.POST
            )
            if form.is_valid():
                subscriber = form.cleaned_data.get('subscriber')
                display_name = form.cleaned_data.get('display_name')
                delivery_mode = form.cleaned_data.get('delivery_mode')
                delivery_status = form.cleaned_data.get('delivery_status')
                response = self.mailing_list.subscribe(
                    subscriber,
                    display_name,
                    pre_verified=True,
                    pre_confirmed=True,
                    delivery_mode=delivery_mode,
                    delivery_status=delivery_status,
                )
                if (
                    type(response) == dict
                    and response.get('token_owner')  # noqa: W504
                    == TokenOwner.moderator
                ):
                    messages.success(
                        request,
                        _(
                            'Your subscription request has been'
                            ' submitted and is waiting for moderator'
                            ' approval.'
                        ),
                    )
                else:
                    messages.success(
                        request,
                        _('You are subscribed to %s.')
                        % self.mailing_list.fqdn_listname,
                    )
            else:
                messages.error(
                    request, _('Something went wrong. Please try again.')
                )
        except HTTPError as e:
            messages.error(request, e.msg)
        return redirect('list_summary', self.mailing_list.list_id)


class ListAnonymousSubscribeView(MailingListView):
    """
    view name: `list_anonymous_subscribe`
    """

    def post(self, request, list_id):
        """
        Subscribes an email address to a mailing list via POST and
        redirects to the `list_summary` view.
        This view is used for unauthenticated users and asks Mailman core to
        verify the supplied email address.
        """
        try:
            form = ListAnonymousSubscribe(request.POST)
            if form.is_valid():
                email = form.cleaned_data.get('email')
                display_name = form.cleaned_data.get('display_name')
                self.mailing_list.subscribe(
                    email,
                    display_name,
                    pre_verified=False,
                    pre_confirmed=False,
                )
                messages.success(
                    request,
                    _('Please check your inbox for ' 'further instructions'),
                )
            else:
                messages.error(
                    request, _('Something went wrong. Please try again.')
                )
        except HTTPError as e:
            messages.error(request, e.msg)
        return redirect('list_summary', self.mailing_list.list_id)


class ListUnsubscribeView(MailingListView):

    """Unsubscribe from a mailing list."""

    @method_decorator(login_required)
    def post(self, request, *args, **kwargs):
        email = request.POST['email']
        # Verify the user actually controls this email, should
        # return 1 if the user owns the email, 0 otherwise.
        found_email = EmailAddress.objects.filter(
            user=request.user, email=email, verified=True
        ).count()
        if found_email == 0:
            messages.error(request, _('You can only unsubscribe yourself.'))
            return redirect('list_summary', self.mailing_list.list_id)
        if self._has_pending_unsub_req(email):
            messages.error(
                request,
                _(
                    'You have a pending unsubscription request waiting for'
                    ' moderator approval.'
                ),
            )
            return redirect('list_summary', self.mailing_list.list_id)
        try:
            response = self.mailing_list.unsubscribe(email)
            if response is not None and response.get('token') is not None:
                messages.success(
                    request,
                    _(
                        'Your unsubscription request has been'
                        ' submitted and is waiting for moderator'
                        ' approval.'
                    ),
                )
            else:
                messages.success(
                    request,
                    _('%s has been unsubscribed' ' from this list.') % email,
                )
        except ValueError as e:
            messages.error(request, e)
        return redirect('list_summary', self.mailing_list.list_id)


@login_required
@list_owner_required
def list_mass_subscribe(request, list_id):
    mailing_list = List.objects.get_or_404(fqdn_listname=list_id)
    if request.method == 'POST':
        form = ListMassSubscription(request.POST)
        failed = []
        if form.is_valid():
            for data in form.cleaned_data['emails']:
                try:
                    # Parse the data to get the address and the display name
                    display_name, address = email.utils.parseaddr(data)
                    validate_email(address)
                    p_v = form.cleaned_data['pre_verified']
                    p_c = form.cleaned_data['pre_confirmed']
                    p_a = form.cleaned_data['pre_approved']
                    inv = form.cleaned_data['invitation']
                    mailing_list.subscribe(
                        address=address,
                        display_name=display_name,
                        pre_verified=p_v,
                        pre_confirmed=p_c,
                        pre_approved=p_a,
                        invitation=inv,
                        send_welcome_message=form.cleaned_data[
                            'send_welcome_message'
                        ],
                    )
                    if inv:
                        message = _(
                            'The address %(address)s has been'
                            ' invited to %(list)s.'
                        )
                    else:
                        message = _(
                            'The address %(address)s has been'
                            ' subscribed to %(list)s.'
                        )
                        if not (p_v and p_c and p_a):
                            message += _(
                                ' Subscription may be pending address'
                                ' verification, confirmation or'
                                ' approval as appropriate.'
                            )
                    messages.success(
                        request,
                        message
                        % {
                            'address': address,
                            'list': mailing_list.fqdn_listname,
                        },
                    )
                except HTTPError as e:
                    messages.error(request, e)
                    failed.append(address)
                except ValidationError:
                    messages.error(
                        request,
                        _('The email address %s' ' is not valid.') % address,
                    )
                    # When the email address is bad, set the focus back to the
                    # email field after returning to the same page.
                    form.fields['emails'].widget.attrs['autofocus'] = True
                    failed.append(address)
        # Make a copy of the initial list of emails and update just the
        # emails to only the failed ones, so they are populated for being
        # fixed by the list owner.
        initial = form.cleaned_data.copy()
        initial['emails'] = failed
        # Re-initialize the form with the data.
        form = ListMassSubscription(initial=initial)
    else:
        form = ListMassSubscription()
    return render(
        request,
        'postorius/lists/mass_subscribe.html',
        {'form': form, 'list': mailing_list},
    )


class ListMassRemovalView(MailingListView):
    """Class For Mass Removal"""

    @method_decorator(login_required)
    @method_decorator(list_owner_required)
    def get(self, request, *args, **kwargs):
        form = ListMassRemoval()
        return render(
            request,
            'postorius/lists/mass_removal.html',
            {'form': form, 'list': self.mailing_list},
        )

    @method_decorator(list_owner_required)
    def post(self, request, *args, **kwargs):
        form = ListMassRemoval(request.POST)
        if not form.is_valid():
            messages.error(request, _('Please fill out the form correctly.'))
        else:
            valid_emails = []
            invalid_emails = []
            for data in form.cleaned_data['emails']:
                try:
                    # Parse the data to get the address.
                    address = email.utils.parseaddr(data)[1]
                    validate_email(address)
                    valid_emails.append(address)
                except ValidationError:
                    invalid_emails.append(data)

            try:
                self.mailing_list.mass_unsubscribe(valid_emails)
                messages.success(
                    request,
                    _(
                        'These addresses {address} have been'
                        ' unsubscribed from {list}.'.format(
                            address=', '.join(valid_emails),
                            list=self.mailing_list.fqdn_listname,
                        )
                    ),
                )
            except (HTTPError, ValueError) as e:
                messages.error(request, e)

            if len(invalid_emails) > 0:
                messages.error(
                    request,
                    _('The email address %s is not valid.') % invalid_emails,
                )
        return redirect('mass_removal', self.mailing_list.list_id)


def _perform_action(message_ids, action):
    for message_id in message_ids:
        action(message_id)


@login_required
@list_moderator_required
def list_moderation(request, list_id, held_id=-1):
    mailing_list = List.objects.get_or_404(fqdn_listname=list_id)
    if request.method == 'POST':
        form = MultipleChoiceForm(request.POST)
        if form.is_valid():
            message_ids = form.cleaned_data['choices']
            try:
                if 'accept' in request.POST:
                    _perform_action(message_ids, mailing_list.accept_message)
                    messages.success(
                        request, _('The selected messages were accepted')
                    )
                elif 'reject' in request.POST:
                    _perform_action(message_ids, mailing_list.reject_message)
                    messages.success(
                        request, _('The selected messages were rejected')
                    )
                elif 'discard' in request.POST:
                    _perform_action(message_ids, mailing_list.discard_message)
                    messages.success(
                        request, _('The selected messages were discarded')
                    )
            except HTTPError:
                messages.error(request, _('Message could not be found'))
    else:
        form = MultipleChoiceForm()
    held_messages = paginate(
        mailing_list.get_held_page,
        request.GET.get('page'),
        request.GET.get('count'),
        paginator_class=MailmanPaginator,
    )
    context = {
        'list': mailing_list,
        'held_messages': held_messages,
        'form': form,
        'ACTION_CHOICES': ACTION_CHOICES,
    }
    return render(request, 'postorius/lists/held_messages.html', context)


@require_http_methods(['POST'])
@login_required
@list_moderator_required
def moderate_held_message(request, list_id):
    """Moderate one held message"""
    mailing_list = List.objects.get_or_404(fqdn_listname=list_id)
    msg = mailing_list.get_held_message(request.POST['msgid'])
    moderation_choice = request.POST.get('moderation_choice')
    reason = request.POST.get('reason')

    try:
        if 'accept' in request.POST:
            mailing_list.accept_message(msg.request_id)
            messages.success(request, _('The message was accepted'))
        elif 'reject' in request.POST:
            mailing_list.reject_message(msg.request_id, reason=reason)
            messages.success(request, _('The message was rejected'))
        elif 'discard' in request.POST:
            mailing_list.discard_message(msg.request_id)
            messages.success(request, _('The message was discarded'))
    except HTTPError as e:
        if e.code == 404:
            messages.error(request, _('Held message was not found.'))
            return redirect('list_held_messages', list_id)
        else:
            raise

    moderation_choices = dict(ACTION_CHOICES)
    if moderation_choice in moderation_choices:
        member = get_member_or_nonmember(mailing_list, msg.sender)
        if member is not None:
            member.moderation_action = moderation_choice
            member.save()
            messages.success(
                request,
                _(
                    'Moderation action for {} set to {}'.format(
                        member, moderation_choices[moderation_choice]
                    )
                ),
            )
        else:
            messages.error(
                request,
                _('Failed to set moderation action for {}'.format(msg.sender)),
            )
    return redirect('list_held_messages', list_id)


@login_required
@list_owner_required
def csv_view(request, list_id):
    """Export all the subscriber in csv"""
    mm_lists = List.objects.get_or_404(fqdn_listname=list_id)

    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="Subscribers.csv"'

    writer = csv.writer(response)
    if mm_lists:
        for i in mm_lists.members:
            writer.writerow([i.email])

    return response


def _get_choosable_domains(request):
    domains = Domain.objects.all()
    return [(d.mail_host, d.mail_host) for d in domains]


def _get_choosable_styles(request):
    styles = Style.objects.all()
    options = [
        (style['name'], style['description']) for style in styles['styles']
    ]
    return options


def _get_default_style():
    return Style.objects.all()['default']


@login_required
@superuser_required
def list_new(request, template='postorius/lists/new.html'):
    """
    Add a new mailing list.
    If the request to the function is a GET request an empty form for
    creating a new list will be displayed. If the request method is
    POST the form will be evaluated. If the form is considered
    correct the list gets created and otherwise the form with the data
    filled in before the last POST request is returned. The user must
    be logged in to create a new list.
    """
    mailing_list = None
    choosable_domains = [('', _('Choose a Domain'))]
    choosable_domains += _get_choosable_domains(request)
    choosable_styles = _get_choosable_styles(request)
    if request.method == 'POST':
        form = ListNew(choosable_domains, choosable_styles, request.POST)
        if form.is_valid():
            # grab domain
            domain = Domain.objects.get_or_404(
                mail_host=form.cleaned_data['mail_host']
            )
            # creating the list
            try:
                mailing_list = domain.create_list(
                    form.cleaned_data['listname'],
                    style_name=form.cleaned_data['list_style'],
                )
                mailing_list.add_owner(form.cleaned_data['list_owner'])
                list_settings = mailing_list.settings
                if form.cleaned_data['description']:
                    list_settings['description'] = form.cleaned_data[
                        'description'
                    ]
                list_settings['advertised'] = form.cleaned_data['advertised']
                list_settings.save()
                messages.success(request, _('List created'))
                mailinglist_created.send(
                    sender=List, list_id=mailing_list.list_id
                )
                return redirect('list_summary', list_id=mailing_list.list_id)
            # TODO catch correct Error class:
            except HTTPError as e:
                # Right now, there is no good way to detect that this is a
                # duplicate mailing list request other than checking the
                # reason for 400 error.
                if e.reason == 'Mailing list exists':
                    form.add_error(
                        'listname', _('Mailing List already exists.')
                    )
                    return render(request, template, {'form': form})
                # Otherwise just render the generic error page.
                return render(
                    request, 'postorius/errors/generic.html', {'error': e.msg}
                )
        else:
            messages.error(request, _('Please check the errors below'))
    else:
        form = ListNew(
            choosable_domains,
            choosable_styles,
            initial={
                'list_owner': request.user.email,
                'advertised': True,
                'list_style': _get_default_style(),
            },
        )
    return render(request, template, {'form': form})


def _unique_lists(lists):
    """Return unique lists from a list of mailing lists."""
    return {mlist.list_id: mlist for mlist in lists}.values()


def _get_mail_host(web_host):
    """Get the mail_host for a web_host if FILTER_VHOST is true and there's
    only one mail_host for this web_host.
    """
    if not getattr(settings, 'FILTER_VHOST', False):
        return None
    mail_hosts = []
    use_web_host = False
    for domain in Domain.objects.all():
        try:
            if (
                MailDomain.objects.get(
                    mail_domain=domain.mail_host
                ).site.domain
                == web_host
            ):
                if domain.mail_host not in mail_hosts:
                    mail_hosts.append(domain.mail_host)
        except MailDomain.DoesNotExist:
            use_web_host = True
    if len(mail_hosts) == 1:
        return mail_hosts[0]
    elif len(mail_hosts) == 0 and use_web_host:
        return web_host
    else:
        return None


@login_required
def list_index_authenticated(request):
    """Index page for authenticated users.

    Index page for authenticated users is slightly different than
    un-authenticated ones. Authenticated users will see all their memberships
    in the index page.

    This view is not paginated and will show all the lists.

    """
    role = request.GET.get('role', None)
    client = get_mailman_client()
    choosable_domains = _get_choosable_domains(request)

    # Get the user_id of the current user
    user_id = get_mailman_user_id(request.user)

    mail_host = _get_mail_host(request.get_host().split(':')[0])
    # Get all the mailing lists for the current user.
    try:
        all_lists = client.find_lists(
            user_id, role=role, mail_host=mail_host, count=sys.maxsize
        )
    except HTTPError:
        # No lists exist with the given role for the given user.
        all_lists = []
    # If the user has no list that they are subscriber/owner/moderator of, we
    # just redirect them to the index page with all lists.
    if len(all_lists) == 0 and role is None:
        return redirect(reverse('list_index') + '?all-lists')
    # Render the list index page with `check_advertised = False` since we don't
    # need to check for advertised list given that all the users are related
    # and know about the existence of the list anyway.
    context = {
        'lists': _unique_lists(all_lists),
        'domain_count': len(choosable_domains),
        'role': role,
        'check_advertised': False,
    }
    return render(request, 'postorius/index.html', context)


def list_index(request, template='postorius/index.html'):
    """Show a table of all public mailing lists."""
    # TODO maxking: Figure out why does this view accept POST request and why
    # can't it be just a GET with list parameter.
    if request.method == 'POST':
        return redirect('list_summary', list_id=request.POST['list'])
    # If the user is logged-in, show them only related lists in the index,
    # except role is present in requests.GET.
    if request.user.is_authenticated and 'all-lists' not in request.GET:
        return list_index_authenticated(request)

    def _get_list_page(count, page):
        client = get_mailman_client()
        advertised = not request.user.is_superuser
        mail_host = _get_mail_host(request.get_host().split(':')[0])
        return client.get_list_page(
            advertised=advertised, mail_host=mail_host, count=count, page=page
        )

    lists = paginate(
        _get_list_page,
        request.GET.get('page'),
        request.GET.get('count'),
        paginator_class=MailmanPaginator,
    )

    # This is just an optimization to skip un-necessary API
    # calls. postorius/index.html page shows the 'Create New Domain' button
    # when the logged-in user is a super user. There is no point making those
    # API calls if the user isn't a Superuser. So, just call the number 0 if
    # the user isn't SU.
    if request.user.is_superuser:
        domain_count = len(_get_choosable_domains(request))
    else:
        domain_count = 0

    return render(
        request,
        template,
        {
            'lists': lists,
            'check_advertised': True,
            'all_lists': True,
            'domain_count': domain_count,
        },
    )


@login_required
@list_owner_required
def list_delete(request, list_id):
    """Deletes a list but asks for confirmation first."""
    the_list = List.objects.get_or_404(fqdn_listname=list_id)
    if request.method == 'POST':
        the_list.delete()
        mailinglist_deleted.send(sender=List, list_id=list_id)
        return redirect('list_index')
    else:
        submit_url = reverse('list_delete', kwargs={'list_id': list_id})
        cancel_url = reverse(
            'list_index',
        )
        return render(
            request,
            'postorius/lists/confirm_delete.html',
            {
                'submit_url': submit_url,
                'cancel_url': cancel_url,
                'list': the_list,
            },
        )


@login_required
@list_moderator_required
def list_pending_confirmations(request, list_id):
    """Shows a list of subscription requests."""
    return _list_subscriptions(
        request=request,
        list_id=list_id,
        token_owner=TokenOwner.subscriber,
        template='postorius/lists/pending_confirmations.html',
        page_title=_('Subscriptions pending user confirmation'),
    )


@login_required
@list_moderator_required
def list_subscription_requests(request, list_id):
    """Shows a list of subscription requests."""
    return _list_subscriptions(
        request=request,
        list_id=list_id,
        token_owner=TokenOwner.moderator,
        template='postorius/lists/subscription_requests.html',
        page_title=_('Subscriptions pending approval'),
    )


@login_required
@list_moderator_required
def list_unsubscription_requests(request, list_id):
    """Shows a list of pending unsubscription requests."""
    return _list_subscriptions(
        request=request,
        list_id=list_id,
        token_owner=TokenOwner.moderator,
        template='postorius/lists/subscription_requests.html',
        page_title=_('Un-subscriptions pending approval'),
        request_type='unsubscription',
    )


def _list_subscriptions(
    request,
    list_id,
    token_owner,
    template,
    page_title,
    request_type='subscription',
):
    m_list = List.objects.get_or_404(fqdn_listname=list_id)
    requests = [
        req
        for req in m_list.get_requests(
            token_owner=token_owner, request_type=request_type
        )
    ]
    paginated_requests = paginate(
        requests, request.GET.get('page'), request.GET.get('count', 25)
    )
    page_subtitle = '(%d)' % len(requests)
    return render(
        request,
        template,
        {
            'list': m_list,
            'paginated_requests': paginated_requests,
            'page_title': page_title,
            'page_subtitle': page_subtitle,
            'request_type': request_type,
        },
    )


@login_required
@list_moderator_required
@require_http_methods(['POST'])
def handle_subscription_request(request, list_id, request_id):
    """
    Handle a subscription request. Possible actions:
        - accept
        - defer
        - reject
        - discard
    """
    confirmation_messages = {
        'accept': _('The request has been accepted.'),
        'reject': _('The request has been rejected.'),
        'discard': _('The request has been discarded.'),
        'defer': _('The request has been defered.'),
    }
    # Get the action by the presence of one of the values in POST data.
    for each in confirmation_messages:
        if each in request.POST:
            action = each
            break
    # If None of the actions were specified, just raise an error and return.
    m_list = List.objects.get_or_404(fqdn_listname=list_id)
    if action is None:
        messages.error(f'Invalid or missing action {action!r}')
        return redirect('list_subscription_requests', m_list.list_id)
    reason = request.POST.get('reason')
    try:
        # Moderate request and add feedback message to session.
        m_list.moderate_request(request_id, action, reason)
        messages.success(request, confirmation_messages[action])
    except HTTPError as e:
        if e.code == 409:
            messages.success(
                request, _('The request was already moderated: %s') % e.reason
            )
        else:
            messages.error(
                request, _('The request could not be moderated: %s') % e.reason
            )
    # Determine the kind of request we handled using the 'kind' POST param.
    req_kind = request.POST.get('kind')
    if req_kind == 'unsubscription':
        return redirect('list_unsubscription_requests', m_list.list_id)
    elif req_kind == 'confirmation':
        return redirect('list_pending_confirmation', m_list.list_id)

    # The default is the list_subscription page.
    return redirect('list_subscription_requests', m_list.list_id)


SETTINGS_SECTION_NAMES = (
    ('list_identity', gettext_lazy('List Identity')),
    ('automatic_responses', gettext_lazy('Automatic Responses')),
    ('alter_messages', gettext_lazy('Alter Messages')),
    ('dmarc_mitigations', gettext_lazy('DMARC Mitigations')),
    ('digest', gettext_lazy('Digest')),
    ('message_acceptance', gettext_lazy('Message Acceptance')),
    ('archiving', gettext_lazy('Archiving')),
    ('subscription_policy', gettext_lazy('Member Policy')),
    ('bounce_processing', gettext_lazy('Bounce Processing')),
)

SETTINGS_FORMS = {
    'list_identity': ListIdentityForm,
    'automatic_responses': ListAutomaticResponsesForm,
    'alter_messages': AlterMessagesForm,
    'dmarc_mitigations': DMARCMitigationsForm,
    'digest': DigestSettingsForm,
    'message_acceptance': MessageAcceptanceForm,
    'archiving': ArchiveSettingsForm,
    'subscription_policy': MemberPolicyForm,
    'bounce_processing': BounceProcessingForm,
}


@login_required
@list_owner_required
def list_settings(
    request,
    list_id=None,
    visible_section=None,
    template='postorius/lists/settings.html',
):
    """
    View and edit the settings of a list.
    The function requires the user to be logged in and have the
    permissions necessary to perform the action.

    Use /<NAMEOFTHESECTION>/<NAMEOFTHEOPTION>
    to show only parts of the settings
    <param> is optional / is used to differ in between section and option might
    result in using //option
    """
    if visible_section is None:
        visible_section = 'list_identity'
    try:
        form_class = SETTINGS_FORMS[visible_section]
    except KeyError:
        raise Http404('No such settings section')
    m_list = List.objects.get_or_404(fqdn_listname=list_id)
    list_settings = m_list.settings
    initial_data = dict((key, value) for key, value in list_settings.items())
    # List settings are grouped and processed in different forms.
    if request.method == 'POST':
        form = form_class(request.POST, mlist=m_list, initial=initial_data)
        if form.is_valid():
            try:
                for key in form.changed_data:
                    if key in form_class.mlist_properties:
                        setattr(m_list, key, form.cleaned_data[key])
                    else:
                        val = form.cleaned_data[key]
                        # Empty list isn't really a valid value and Core
                        # interprets empty string as empty value for
                        # ListOfStringsField. We are doing it here instead of
                        # ListOfStringsField.to_python() because output from
                        # list of strings is expected to be a list and the NULL
                        # value is hence an empty list. The serialization of
                        # the empty list is for us empty string, hence we are
                        # doing this here. Technically, it can be done outside
                        # of this view, but there aren't any other use cases
                        # where we'd want an empty list of strings.
                        if val == []:
                            val = ''
                        list_settings[key] = val
                list_settings.save()
                messages.success(request, _('The settings have been updated.'))
                mailinglist_modified.send(sender=List, list_id=m_list.list_id)
            except HTTPError as e:
                messages.error(request, _('An error occurred: ') + e.reason)
            return redirect('list_settings', m_list.list_id, visible_section)
    else:
        form = form_class(initial=initial_data, mlist=m_list)

    return render(
        request,
        template,
        {
            'form': form,
            'section_names': SETTINGS_SECTION_NAMES,
            'list': m_list,
            'visible_section': visible_section,
        },
    )


@login_required
@list_owner_required
def remove_role(
    request,
    list_id=None,
    role=None,
    address=None,
    template='postorius/lists/confirm_remove_role.html',
):
    """Removes a list moderator or owner."""
    the_list = List.objects.get_or_404(fqdn_listname=list_id)
    redirect_on_success = redirect('list_members', the_list.list_id, role)
    roster = getattr(the_list, '{}s'.format(role))
    all_emails = [each.email for each in roster]
    if address not in all_emails:
        messages.error(
            request,
            _('The user %(email)s is not in the %(role)s group')
            % {'email': address, 'role': role},
        )
        return redirect('list_members', the_list.list_id, role)

    if role == 'owner':
        if len(roster) == 1:
            messages.error(request, _('Removing the last owner is impossible'))
            return redirect('list_members', the_list.list_id, role)
        user_emails = (
            EmailAddress.objects.filter(user=request.user, verified=True)
            .order_by('email')
            .values_list('email', flat=True)
        )
        if address in user_emails:
            # The user is removing themselves, redirect to the list info page
            # because they won't have access to the members page anyway.
            redirect_on_success = redirect('list_summary', the_list.list_id)

    if request.method == 'POST':
        try:
            the_list.remove_role(role, address)
        except HTTPError as e:
            messages.error(
                request,
                _('The user could not be removed: %(msg)s') % {'msg': e.msg},
            )
            return redirect('list_members', the_list.list_id, role)
        messages.success(
            request,
            _(
                'The user %(address)s has been removed'
                ' from the %(role)s group.'
            )
            % {'address': address, 'role': role},
        )
        return redirect_on_success
    return render(
        request,
        template,
        {'role': role, 'address': address, 'list_id': the_list.list_id},
    )


@login_required
@list_owner_required
def remove_all_subscribers(request, list_id):
    """Empty the list by unsubscribing all members."""

    mlist = List.objects.get_or_404(fqdn_listname=list_id)
    if len(mlist.members) == 0:
        messages.error(
            request, _('No member is subscribed to the list currently.')
        )
        return redirect('mass_removal', mlist.list_id)
    if request.method == 'POST':
        try:
            # TODO maxking: This doesn't scale. Either the Core should provide
            # an endpoint to remove all subscribers or there should be some
            # better way to do this. Maybe, Core can take a list of email
            # addresses in batches of 50 and unsubscribe all of them.
            for names in mlist.members:
                mlist.unsubscribe(names.email)
            messages.success(
                request,
                _('All members have been' ' unsubscribed from the list.'),
            )
            return redirect('list_members', mlist.list_id, 'subscriber')
        except Exception as e:
            messages.error(request, e)
    return render(
        request,
        'postorius/lists/confirm_removeall_subscribers.html',
        {'list': mlist},
    )


@login_required
@list_owner_required
def list_bans(request, list_id):
    return bans_view(
        request, list_id=list_id, template='postorius/lists/bans.html'
    )


@login_required
@list_owner_required
def list_header_matches(request, list_id):
    """
    View and edit the list's header matches.
    """
    m_list = List.objects.get_or_404(fqdn_listname=list_id)
    header_matches = m_list.header_matches
    HeaderMatchFormset = formset_factory(
        ListHeaderMatchForm,
        extra=1,
        can_delete=True,
        can_order=True,
        formset=ListHeaderMatchFormset,
    )
    initial_data = [
        dict(
            [
                (key, getattr(hm, key))
                for key in ListHeaderMatchForm.base_fields
            ]
        )
        for hm in header_matches
    ]

    # Process form submission.
    if request.method == 'POST':
        formset = HeaderMatchFormset(request.POST, initial=initial_data)
        if formset.is_valid():
            if not formset.has_changed():
                return redirect('list_header_matches', list_id)
            # Purge the existing header_matches
            header_matches.clear()
            # Add the ones in the form

            def form_order(f):
                # If ORDER is None (new header match), add it last.
                return f.cleaned_data.get('ORDER') or len(formset.forms)

            errors = []
            for form in sorted(formset, key=form_order):
                if 'header' not in form.cleaned_data:
                    # The new header match form was not filled
                    continue
                if form.cleaned_data.get('DELETE'):
                    continue
                try:
                    header_matches.add(
                        header=form.cleaned_data['header'],
                        pattern=form.cleaned_data['pattern'],
                        action=form.cleaned_data['action'],
                    )
                except HTTPError as e:
                    errors.append(e)
            for e in errors:
                messages.error(request, _('An error occurred: %s') % e.reason)
            if not errors:
                messages.success(
                    request,
                    _('The header matches were' ' successfully modified.'),
                )
            return redirect('list_header_matches', list_id)
    else:
        formset = HeaderMatchFormset(initial=initial_data)
    # Adapt the last form to create new matches
    form_new = formset.forms[-1]
    form_new.fields['header'].widget.attrs['placeholder'] = _('New header')
    form_new.fields['pattern'].widget.attrs['placeholder'] = _('New pattern')
    del form_new.fields['ORDER']
    del form_new.fields['DELETE']

    return render(
        request,
        'postorius/lists/header_matches.html',
        {
            'list': m_list,
            'formset': formset,
        },
    )


def confirm_token(request, list_id):
    """Confirm token sent via Mailman Core.

    This view allows confirming tokens sent via Mailman Core for confirming
    subscriptions or verifying email addresses. This is an un-authenticated
    view and the authentication is done by assuming the secrecy of the token
    sent.

    :param request: The Django request object.
    :param list_id: The current mailinglist id.

    """
    m_list = List.objects.get_or_404(fqdn_listname=list_id)
    if request.method == 'POST':
        form = TokenConfirmForm(request.POST)
        # If the token is None or something, just raise 400 error.
        if not form.is_valid():
            raise HTTPError(
                request.path, 400, _('Invalid confirmation token'), None, None
            )
        token = form.cleaned_data.get('token')
        try:
            pending_req = m_list.get_request(token)
        except HTTPError as e:
            if e.code == 404:
                raise HTTPError(
                    request.path,
                    404,
                    _('Token expired or invalid.'),
                    None,
                    None,
                )
            raise
        # Since we only accept the token there isn't any need for Form data. We
        # just need a POST request at this URL to accept the token.
        m_list.moderate_request(token, action='accept')
        return redirect('list_summary', m_list.list_id)
    # Get the token from url parameter.
    token = request.GET.get('token')
    try:
        pending_req = m_list.get_request(token)
    except HTTPError as e:
        if e.code == 404:
            raise HTTPError(
                request.path, 404, _('Token expired or invalid.'), None, None
            )
        raise

    # If this is a token pending moderator approval, they need to login and
    # approve it from the pending requests page.
    if pending_req.get('token_owner') == 'moderator':
        return redirect('list_subscription_requests', m_list.list_id)

    token_type = pending_req.get('type')
    form = TokenConfirmForm(initial=dict(token=token))
    # Show the display_name if it is not None or "".
    if pending_req.get('display_name'):
        addr = (
            pending_req.get('display_name')
            + ' <'
            + pending_req.get('email')
            + '>'
        )
    else:
        addr = pending_req.get('email')

    return render(
        request,
        'postorius/lists/confirm_token.html',
        {
            'mlist': m_list,
            'addr': addr,
            'token': token,
            'type': token_type,
            'form': form,
        },
    )



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

import json
from email import message_from_string, policy
from email.parser import HeaderParser

from django.contrib.auth.decorators import login_required
from django.http import Http404, HttpResponse
from django.urls import reverse
from django.utils.translation import gettext as _

from django_mailman3.lib.scrub import Scrubber

from postorius.auth.decorators import list_moderator_required
from postorius.models import List


def parse(message):
    msgobj = message_from_string(message, policy=policy.SMTP)
    header_parser = HeaderParser()

    headers = []
    headers_dict = header_parser.parsestr(message)
    for key in headers_dict.keys():
        headers.append('{}: {}'.format(key, headers_dict[key]))
    content = Scrubber(msgobj).scrub()[0]
    return {
        'body': content,
        'headers': '\n'.join(headers),
    }


def get_attachments(message):
    message = message_from_string(message, policy=policy.SMTP)
    return Scrubber(message).scrub()[1]


@login_required
@list_moderator_required
def get_held_message(request, list_id, held_id=-1):
    """Return a held message as a json object"""
    if held_id == -1:
        raise Http404(_('Message does not exist'))

    held_message = List.objects.get_or_404(
        fqdn_listname=list_id
    ).get_held_message(held_id)
    if 'raw' in request.GET:
        return HttpResponse(held_message.msg, content_type='text/plain')
    response_data = dict()
    response_data['sender'] = held_message.sender
    response_data['subject'] = held_message.subject
    response_data['reason'] = held_message.reason
    response_data['hold_date'] = held_message.hold_date
    response_data['msg'] = parse(held_message.msg)
    response_data['msgid'] = held_message.request_id
    response_data['attachments'] = []
    attachments = get_attachments(held_message.msg)
    for attachment in attachments:
        counter, name, content_type, encoding, content = attachment
        response_data['attachments'].append(
            (
                reverse(
                    'rest_attachment_for_held_message',
                    args=(list_id, held_id, counter),
                ),
                name,
            )
        )

    return HttpResponse(
        json.dumps(response_data), content_type='application/json'
    )


@login_required
@list_moderator_required
def get_attachment_for_held_message(request, list_id, held_id, attachment_id):
    held_message = List.objects.get_or_404(
        fqdn_listname=list_id
    ).get_held_message(held_id)
    attachments = get_attachments(held_message.msg)
    for attachment in attachments:
        if attachment[0] == int(attachment_id):
            response = HttpResponse(attachment[4], content_type=attachment[2])
            response[
                'Content-Disposition'
            ] = 'attachment;filename="{}"'.format(attachment[1])
            return response
    raise Http404(_('Attachment does not exist'))



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

import re

from django import forms
from django.apps import apps
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.utils.translation import gettext_lazy as _

from django_mailman3.lib.mailman import get_mailman_client

from postorius.forms.fields import (
    ACTION_CHOICES,
    ListOfStringsField,
    delivery_mode_field,
    delivery_status_field,
    moderation_action_field,
)
from postorius.forms.validators import validate_uuid_or_email
from postorius.models import EmailTemplate, _email_template_help_text
from postorius.utils import LANGUAGES


DIGEST_FREQUENCY_CHOICES = (
    ('daily', _('Daily')),
    ('weekly', _('Weekly')),
    ('quarterly', _('Quarterly')),
    ('monthly', _('Monthly')),
    ('yearly', _('Yearly')),
)

ROSTER_VISIBILITY_CHOICES = (
    ('moderators', _('Only mailinglist administrators')),
    ('members', _('Mailinglist administrators and members')),
    ('public', _('Anyone')),
)


EMPTY_STRING = ''


class ListNew(forms.Form):

    """
    Form fields to add a new list. Languages are hard coded which should
    be replaced by a REST lookup of available languages.
    """

    listname = forms.CharField(
        label=_('List Name'),
        required=True,
        error_messages={
            'required': _('Please enter a name for your list.'),
            'invalid': _('Please enter a valid list name.'),
        },
    )
    mail_host = forms.ChoiceField()
    list_owner = forms.EmailField(
        label=_('Initial list owner address'),
        error_messages={
            'required': _("Please enter the list owner's email address.")
        },
        required=True,
    )
    advertised = forms.ChoiceField(
        widget=forms.RadioSelect(),
        label=_('Advertise this list?'),
        error_messages={'required': _('Please choose a list type.')},
        required=True,
        choices=(
            (True, _('Advertise this list in list index')),
            (False, _('Hide this list in list index')),
        ),
    )
    list_style = forms.ChoiceField()
    description = forms.CharField(label=_('Short Description'), required=False)

    def __init__(self, domain_choices, style_choices, *args, **kwargs):
        super(ListNew, self).__init__(*args, **kwargs)
        self.fields['mail_host'] = forms.ChoiceField(
            widget=forms.Select(),
            label=_('Mail Host'),
            required=True,
            choices=domain_choices,
            error_messages={
                'required': _('Choose an existing Domain.'),
                'invalid': _('Choose a valid Mail Host'),
            },
        )
        self.fields['list_style'] = forms.ChoiceField(
            widget=forms.Select(),
            label=_('List Style'),
            required=True,
            choices=style_choices,
            error_messages={
                'required': _('Choose a List Style.'),
                'invalid': _('Choose a valid List Style.'),
            },
        )
        if len(domain_choices) < 2:
            self.fields['mail_host'].help_text = _(
                'Site admin has not created any domains'
            )
            # if len(choices) < 2:
            #    help_text=_("No domains available: " +
            #                "The site admin must create new domains " +
            #                "before you will be able to create a list")

    def clean_listname(self):
        try:
            validate_email(self.cleaned_data['listname'] + '@example.net')
        except ValidationError:
            # TODO (maxking): Error should atleast point to what is a valid
            # listname. It may not always be obvious which characters aren't
            # allowed in a listname.
            raise forms.ValidationError(_('Please enter a valid listname'))
        return self.cleaned_data['listname']

    class Meta:

        """
        Class to handle the automatic insertion of fieldsets and divs.

        To use it: add a list for each wished fieldset. The first item in
        the list should be the wished name of the fieldset, the following
        the fields that should be included in the fieldset.
        """

        layout = [
            [
                'List Details',
                'listname',
                'mail_host',
                'list_style',
                'list_owner',
                'description',
                'advertised',
            ],
        ]


class ListSubscribe(forms.Form):
    """Form fields to join an existing list."""

    DELIVERY_STATUS_CHOICES = (
        ('enabled', _('Enabled')),
        ('by_user', _('Disabled')),
    )

    subscriber = forms.ChoiceField(
        label=_('Your email address'),
        widget=forms.Select(),
        validators=[
            validate_uuid_or_email,
        ],
        help_text=_(
            'Subscribing via "Primary Address" will change subscription'
            ' address when you change your primary address.'
        ),
        error_messages={
            'required': _('Please enter an email address.'),
            'invalid': _('Please enter a valid email address.'),
        },
    )

    display_name = forms.CharField(
        label=_('Your name (optional)'), required=False
    )

    delivery_mode = delivery_mode_field(default='regular')
    delivery_status = delivery_status_field(
        choices=DELIVERY_STATUS_CHOICES, widget=forms.RadioSelect
    )

    def __init__(self, user_emails, user_id, primary_email, *args, **kwargs):
        super(ListSubscribe, self).__init__(*args, **kwargs)
        choices = list((address, address) for address in user_emails)
        if primary_email and user_id:
            choices.insert(
                0, (user_id, _('Primary Address ({})').format(primary_email))
            )
        self.fields['subscriber'].choices = choices


class ListAnonymousSubscribe(forms.Form):
    """Form fields to join an existing list as an anonymous user."""

    email = forms.CharField(
        label=_('Your email address'),
        validators=[validate_email],
        error_messages={
            'required': _('Please enter an email address.'),
            'invalid': _('Please enter a valid email address.'),
        },
    )

    display_name = forms.CharField(
        label=_('Your name (optional)'), required=False
    )


class ListSettingsForm(forms.Form):
    """
    Base class for list settings forms.
    """

    mlist_properties = []

    def __init__(self, *args, **kwargs):
        self._mlist = kwargs.pop('mlist')
        super(ListSettingsForm, self).__init__(*args, **kwargs)


SUBSCRIPTION_POLICY_CHOICES = (
    ('open', _('Open')),
    ('confirm', _('Confirm')),
    ('moderate', _('Moderate')),
    ('confirm_then_moderate', _('Confirm, then moderate')),
)


class MemberPolicyForm(ListSettingsForm):
    """
    Policies related to members.
    """

    subscription_policy = forms.ChoiceField(
        label=_('Subscription Policy'),
        choices=SUBSCRIPTION_POLICY_CHOICES,
        help_text=_(
            'Open: Subscriptions are added automatically\n'
            'Confirm: Subscribers need to confirm the subscription '
            'using an email sent to them\n'
            'Moderate: Moderators will have to authorize '
            'each subscription manually.\n'
            'Confirm then Moderate: First subscribers have to confirm,'
            ' then a moderator needs to authorize.'
        ),
    )

    unsubscription_policy = forms.ChoiceField(
        label=_('Un-Subscription Policy'),
        choices=SUBSCRIPTION_POLICY_CHOICES,
        help_text=_(
            'Open: Un-Subscriptions happen automatically\n'
            'Confirm: Subscribers need to confirm the un-subscription '
            'using an email sent to them\n'
            'Moderate: Moderators will have to authorize '
            'each un-subscription manually.\n'
            'Confirm then Moderate: First subscribers have to confirm,'
            ' then a moderator needs to authorize.'
        ),
    )


class BounceProcessingForm(ListSettingsForm):
    """List's bounce processing settings."""

    forward_unrecognized_choices = (
        ('discard', _('Discard')),
        ('administrators', _('List Admins')),
        ('site_owner', _('Site Admin')),
    )

    process_bounces = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Process Bounces'),
        help_text=_(
            'Specifies whether or not this list should do automatic'
            ' bounce processing.'
        ),
    )

    bounce_score_threshold = forms.IntegerField(
        min_value=0,
        label=_('Bounce score threshold'),
        required=False,
        help_text=_(
            "This is the bounce score above which a member's subscription "
            ' will be automatically disabled. When the subscription is '
            ' re-enabled, their bounce score will be reset to zero.'
        ),
    )

    bounce_info_stale_after = forms.CharField(
        label=_('Bounce info stale after'),
        required=False,
        help_text=_(
            "The number of days after which a member's bounce information"
            ' is considered stale. If no new bounces have been received in'
            ' the interim, the bounce score is reset to zero.'
            ' This value must be an integer. '
        ),
    )

    bounce_notify_owner_on_bounce_increment = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Notify owner on bounce increment'),
        help_text=_(
            'This option controls whether or not the list owner is notified'
            " when a member's bounce score is incremented, but to a value"
            ' less than their bounce threshold. '
        ),
    )

    bounce_notify_owner_on_disable = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Notify owner on disable'),
        help_text=_(
            'This option controls whether or not the list owner is notified'
            " when a member's subscription is automatically disabled due"
            ' to their bounce threshold being reached. '
        ),
    )

    bounce_notify_owner_on_removal = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Notify owner on removal'),
        help_text=_(
            'This option controls whether or not the list owner is '
            'notified when a member is removed from the list after '
            'their disabled notifications have been exhausted. '
        ),
    )

    forward_unrecognized_bounces_to = forms.ChoiceField(
        choices=forward_unrecognized_choices,
        widget=forms.RadioSelect,
        label=_('Forward unrecognized bounces'),
        help_text=_(
            'Discard: Unrecognized bounces will be discarded\n'
            'List Admins: Send to the list owners and moderators\n'
            "Site Admin: Send to the site's configured site_owner"
        ),
    )

    bounce_you_are_disabled_warnings_interval = forms.CharField(
        label=_('Bounce disabled warnings interval'),
        required=False,
        help_text=_('The number of days between each disabled notification.'),
    )

    bounce_you_are_disabled_warnings = forms.IntegerField(
        min_value=0,
        label=_('Bounce disable warnings'),
        required=False,
        help_text=_(
            'The number of notices a disabled member will receive before'
            " their address is removed from the mailing list's roster. "
            'Set this to 0 to immediately remove an address from the list'
            ' once their bounce score exceeds the threshold. '
            'This value must be an integer. '
        ),
    )


class ArchiveSettingsForm(ListSettingsForm):
    """
    Set the general archive policy.
    """

    mlist_properties = ['archivers']

    archive_policy_choices = (
        ('public', _('Public archives')),
        ('private', _('Private archives')),
        ('never', _('Do not archive this list')),
    )

    archive_rendering_mode_choices = (
        ('text', _('Plain text')),
        ('markdown', _('Markdown text')),
    )

    archive_policy = forms.ChoiceField(
        choices=archive_policy_choices,
        widget=forms.RadioSelect,
        label=_('Archive policy'),
        help_text=_('Policy for archiving messages for this list'),
    )

    archivers = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple,
        label=_('Active archivers'),
        required=False,
    )  # May be empty if no archivers are desired.

    archive_rendering_mode = forms.ChoiceField(
        choices=archive_rendering_mode_choices,
        widget=forms.RadioSelect,
        required=False,
        label=_('Archive Rendering mode'),
        help_text=_(
            'This option enables rendering of emails in archiver as '
            'rich text with formatting based on markup in the email.'
            '\nCurrently, this option is only supported by Hyperkitty.'
        ),
    )

    def __init__(self, *args, **kwargs):
        super(ArchiveSettingsForm, self).__init__(*args, **kwargs)
        archiver_opts = sorted(self._mlist.archivers.keys())
        self.fields['archivers'].choices = sorted(
            [(key, key) for key in archiver_opts]
        )
        if self.initial:
            self.initial['archivers'] = [
                key
                for key in archiver_opts
                if self._mlist.archivers[key] is True
            ]   # noqa
        # If Hyperkitty isn't installed, do not show the archive_rendering_mode
        # field since it doesn't apply to other archivers.
        if not apps.is_installed('hyperkitty'):
            del self.fields['archive_rendering_mode']

    def clean_archivers(self):
        result = {}
        for archiver, etc in self.fields['archivers'].choices:
            result[archiver] = archiver in self.cleaned_data['archivers']
        self.cleaned_data['archivers'] = result
        return result


class MessageAcceptanceForm(ListSettingsForm):
    """
    List messages acceptance settings.
    """

    acceptable_aliases = ListOfStringsField(
        label=_('Acceptable aliases'),
        required=False,
        help_text=_(
            'This is a list, one per line, of addresses and regexps matching '
            'addresses that are acceptable in To: or Cc: in lieu of the list '
            "posting address when `require_explicit_destination' is enabled. "
            ' Entries are either email addresses or regexps matching email '
            "addresses.  Regexps are entries beginning with `^' and are "
            'matched against every recipient address in the message. The '
            "matching is performed with Python's re.match() function, meaning"
            ' they are anchored to the start of the string.'
        ),
    )
    require_explicit_destination = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Require Explicit Destination'),
        help_text=_(
            'This checks to ensure that the list posting address or an '
            'acceptable alias explicitly appears in a To: or Cc: header in '
            'the post.'
        ),
    )
    administrivia = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Administrivia'),
        help_text=_(
            "Administrivia tests will check postings to see whether it's "
            'really meant as an administrative request (like subscribe, '
            'unsubscribe, etc), and will add it to the administrative '
            'requests queue, notifying the administrator of the new request, '
            'in the process.'
        ),
    )
    default_member_action = forms.ChoiceField(
        widget=forms.RadioSelect(),
        label=_('Default action to take when a member posts to the list'),
        error_messages={
            'required': _('Please choose a default member action.')
        },
        required=True,
        choices=ACTION_CHOICES,
        help_text=_(
            'Default action to take when a member posts to the list.\n'
            'Hold: This holds the message for approval by the list '
            'moderators.\n'
            'Reject: this automatically rejects the message by sending a '
            "bounce notice to the post's author. The text of the bounce "
            'notice can be configured by you.\n'
            'Discard: this simply discards the message, with no notice '
            "sent to the post's author.\n"
            'Accept: accepts any postings without any further checks.\n'
            'Default Processing: run additional checks and accept '
            'the message.'
        ),
    )
    default_nonmember_action = forms.ChoiceField(
        widget=forms.RadioSelect(),
        label=_('Default action to take when a non-member posts to the list'),
        error_messages={
            'required': _('Please choose a default non-member action.')
        },
        required=True,
        choices=ACTION_CHOICES,
        help_text=_(
            "When a post from a non-member is received, the message's sender "
            'is matched against the list of explicitly accepted, held, '
            'rejected (bounced), and discarded addresses. '
            'If no match is found, then this action is taken.'
        ),
    )
    emergency = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Emergency Moderation'),
        help_text=_(
            'When this option is enabled, all list traffic is emergency'
            ' moderated, i.e. held for moderation. Turn this option on when'
            ' your list is experiencing a flamewar and you want a cooling off'
            ' period. '
        ),
    )
    max_message_size = forms.IntegerField(
        min_value=0,
        label=_('Maximum message size'),
        required=False,
        help_text=_(
            'The maximum allowed message size in KB. '
            'This can be used to prevent emails with large attachments. '
            'A size of 0 disables the check.'
        ),
    )
    max_num_recipients = forms.IntegerField(
        min_value=0,
        label=_('Maximum number of recipients'),
        required=False,
        help_text=_(
            'If a post has this many or more explicit recipients (To: and '
            'Cc:), the post will be held for moderation. '
            'This can be used to prevent mass mailings from being accepted. '
            'A value of 0 disables the check.'
        ),
    )

    # TODO: Expose after this functionality actually works in Core.
    # max_days_to_hold = forms.IntegerField(
    #     min_value=0,
    #     label=_('Discard held posts after'),
    #     required=False,
    #     help_text=_(
    #         'No. of days after which held messages will be automatically'
    #         ' discarded.'))

    accept_these_nonmembers = ListOfStringsField(
        label=_('Accept these non-members'),
        required=False,
        help_text=_(
            'This is a list, one per line, of regexps matching '
            'addresses that are allowed to post to this mailing list without'
            ' subscribing to the list.'
            " Entries are regexps beginning with `^' and are matched against"
            ' the sender addresses in the message.'
            ' While non-regexp addresses can be entered here, it is preferred'
            " to add the address as a nonmember and set the nonmember's "
            'Moderation to Default Processing.'
        ),
    )

    hold_these_nonmembers = ListOfStringsField(
        label=_('Hold these non-members'),
        required=False,
        help_text=_(
            'This is a list, one per line, of regexps matching '
            'nonmember addresses, posts from which are held automatically.'
            " Entries are regexps beginning with `^' and are matched against"
            ' the sender addresses in the message.'
            ' While non-regexp addresses can be entered here, it is preferred'
            " to add the address as a nonmember and set the nonmember's "
            'Moderation to Hold.'
        ),
    )

    reject_these_nonmembers = ListOfStringsField(
        label=_('Reject these non-members'),
        required=False,
        help_text=_(
            'This is a list, one per line, of regexps matching '
            'nonmember addresses, posts from which are rejected with notice to'
            ' the sender.'
            " Entries are regexps beginning with `^' and are matched against"
            ' the sender addresses in the message.'
            ' While non-regexp addresses can be entered here, it is preferred'
            " to add the address as a nonmember and set the nonmember's "
            'Moderation to Reject.'
        ),
    )

    discard_these_nonmembers = ListOfStringsField(
        label=_('Discard these non-members'),
        required=False,
        help_text=_(
            'This is a list, one per line, of regexps matching '
            'nonmember addresses, posts from which are discarded automatically'
            ". Entries are regexps beginning with `^' and are matched against"
            ' the sender addresses in the message.'
            ' While non-regexp addresses can be entered here, it is preferred'
            " to add the address as a nonmember and set the nonmember's "
            'Moderation to Discard.'
        ),
    )

    def clean_acceptable_aliases(self):
        # python's urlencode will drop this attribute completely if an empty
        # list is passed with doseq=True. To make it work for us, we instead
        # use an empty string to signify an empty value. In turn, Core will
        # also consider an empty value to be empty list for list-of-strings
        # field.
        if not self.cleaned_data['acceptable_aliases']:
            return EMPTY_STRING
        for alias in self.cleaned_data['acceptable_aliases']:
            if alias.startswith('^'):
                try:
                    re.compile(alias)
                except re.error as e:
                    raise forms.ValidationError(
                        _('Invalid alias regexp: {}: {}').format(alias, e.msg)
                    )
            else:
                try:
                    validate_email(alias)
                except ValidationError:
                    raise forms.ValidationError(
                        _('Invalid alias email: {}').format(alias)
                    )
        return self.cleaned_data['acceptable_aliases']


class DigestSettingsForm(ListSettingsForm):
    """
    List digest settings.
    """

    digests_enabled = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Enable Digests'),
        help_text=_('Should Mailman enable digests for this MailingList?'),
    )
    digest_send_periodic = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Send Digest Periodically'),
        help_text=_('Should Mailman send out digests periodically?'),
    )
    digest_volume_frequency = forms.ChoiceField(
        choices=DIGEST_FREQUENCY_CHOICES,
        widget=forms.RadioSelect,
        required=False,
        label=_('Digest Volume Frequency'),
        help_text=_(
            'At what frequency should Mailman increment the digest '
            'volume number and reset the issue number?'
        ),
    )
    digest_size_threshold = forms.DecimalField(
        label=_('Digest size threshold'),
        help_text=_(
            'How big in Kb should a digest be before ' 'it gets sent out?'
        ),
    )


class DMARCMitigationsForm(ListSettingsForm):
    """
    DMARC Mitigations list settings.
    """

    dmarc_mitigate_action = forms.ChoiceField(
        label=_('DMARC mitigation action'),
        widget=forms.Select(),
        required=False,
        error_messages={
            'required': _('Please choose a DMARC mitigation action.')
        },
        choices=(
            ('no_mitigation', _('No DMARC mitigations')),
            ('munge_from', _('Replace From: with list address')),
            (
                'wrap_message',
                _('Wrap the message in an outer message From: the list.'),
            ),
            ('reject', _('Reject the message')),
            ('discard', _('Discard the message')),
        ),
        help_text=_(
            'The action to apply to messages From: a domain publishing a '
            'DMARC policy of reject or quarantine or to all messages if '
            'DMARC Mitigate unconditionally is True.'
        ),
    )
    dmarc_mitigate_unconditionally = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('DMARC Mitigate unconditionally'),
        help_text=_(
            'If DMARC mitigation action is munge_from or wrap_message, '
            'should it apply to all messages regardless of the DMARC policy '
            'of the From: domain.'
        ),
    )
    dmarc_addresses = ListOfStringsField(
        label=_('DMARC Addresses'),
        required=False,
        help_text=_(
            'Email addresses and regexps matching email addresses, one per '
            "line. Regexps begin with '^' as in '^.*@example\\.com'. DMARC "
            'mitigation action will apply to posts From: an address matching, '
            'case insensitevely, one of these addresses or patterns '
            "regardless of the From: domain's DMARC policy."
        ),
    )
    dmarc_moderation_notice = forms.CharField(
        label=_('DMARC rejection notice'),
        required=False,
        widget=forms.Textarea(),
        help_text=_(
            'Text to replace the default reason in any rejection notice to '
            'be sent when DMARC mitigation action of reject applies.'
        ),
    )
    dmarc_wrapped_message_text = forms.CharField(
        label=_('DMARC wrapped message text'),
        required=False,
        widget=forms.Textarea(),
        help_text=_(
            'Text to be added as a separate text/plain MIME part preceding '
            'the original message part in the wrapped message when DMARC '
            'mitigation action of wrap message applies.'
        ),
    )


PERSONALIZATION_CHOICES = (
    ('none', _('None')),
    ('individual', _('Individual')),
    ('full', _('Full')),
)

PERSONALIZATION_CHOICES_HELP = _(
    """
    None: No personalization.

    Individual: Everyone gets a unique copy of the message, and there are a \
          few more substitution variables, but no headers are modified.

    Full: All of the 'individual' personalization plus recipient header \
          modification. """
)

FILTER_ACTION_CHOICES = (
    ('discard', _('Discard')),
    ('reject', _('Reject')),
    ('forward', _('Forward')),
    ('preserve', _('Preserve')),
)

FILTER_ACTION_HELP = _(
    """Action to take on messages which have no content
after filtering.
 Discard = silently discard the message.
 Reject = discard the message and notify the sender.
 Forward = forward the message to the list owner(s).
 Preserve = save the message in qfiles/bad.
"""
)


class AlterMessagesForm(ListSettingsForm):
    """
    Alter messages list settings.
    """

    personalize = forms.ChoiceField(
        choices=PERSONALIZATION_CHOICES,
        widget=forms.RadioSelect,
        required=False,
        label=_('Personalize'),
        help_text=PERSONALIZATION_CHOICES_HELP,
    )
    anonymous_list = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Anonymous list'),
        help_text=_(
            'Hide the sender of a message, '
            'replacing it with the list address '
            '(Removes From, Sender and Reply-To fields)'
        ),
    )
    include_rfc2369_headers = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Include RFC2369 headers'),
        help_text=_(
            'Yes is highly recommended. RFC 2369 defines a set of List-* '
            'headers that are normally added to every message sent to the '
            'list membership. These greatly aid end-users who are using '
            'standards compliant mail readers. They should normally always '
            'be enabled. However, not all mail readers are standards '
            'compliant yet, and if you have a large number of members who are '
            'using non-compliant mail readers, they may be annoyed at these '
            'headers. You should first try to educate your members as to why '
            'these headers exist, and how to hide them in their mail clients. '
            'As a last resort you can disable these headers, but this is not '
            'recommended (and in fact, your ability to disable these headers '
            'may eventually go away).'
        ),
    )
    allow_list_posts = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Include the list post header'),
        help_text=_(
            'This can be set to no for announce lists that do not wish to '
            'include the List-Post header because posting to the list is '
            'discouraged.'
        ),
    )
    reply_to_address = forms.CharField(
        label=_('Explicit reply-to address'),
        required=False,
        help_text=_(
            'This option allows admins to set an explicit Reply-to address. '
            'It is only used if the reply-to is set to use an explicitly set '
            'header'
        ),
    )
    first_strip_reply_to = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        help_text=_(
            'Should any existing Reply-To: header found in the original '
            'message be stripped? If so, this will be done regardless of '
            'whether an explict Reply-To: header is added by Mailman or not.'
        ),
    )
    reply_goes_to_list = forms.ChoiceField(
        label=_('Reply goes to list'),
        widget=forms.Select(),
        required=False,
        error_messages={'required': _('Please choose a reply-to action.')},
        choices=(
            ('no_munging', _('No Munging')),
            ('point_to_list', _('Reply goes to list')),
            ('explicit_header', _('Explicit Reply-to header set')),
            ('explicit_header_only', _('Explicit Reply-to set; no Cc added')),
        ),
        help_text=_(
            'Where are replies to list messages directed? No Munging is '
            'strongly recommended for most mailing lists. \nThis option '
            'controls what Mailman does to the Reply-To: header in messages '
            'flowing through this mailing list. When set to No Munging, no '
            'Reply-To: header is '
            'added by Mailman, although if one is present in the original '
            'message, it is not stripped. Setting this value to either Reply '
            'to List, Explicit Reply, or Reply Only causes Mailman to insert '
            'a specific Reply-To: header in all messages, overriding the '
            'header in the original message if necessary '
            '(Explicit Reply inserts the value of reply_to_address). '
            'Explicit Reply-to set; no Cc added is useful for'
            'announce-only lists where you want to avoid someone replying '
            'to the list address. There are many reasons not to introduce or '
            'override the Reply-To: header. One is that some posters depend '
            'on their own Reply-To: settings to convey their valid return '
            'address. Another is that modifying Reply-To: makes it much more '
            'difficult to send private replies. See <a href="'
            'http://marc.merlins.org/netrants/reply-to-harmful.html">'
            "`Reply-To' Munging Considered Harmful</a> for a general "
            'discussion of this issue. See <a href="'
            'http://marc.merlins.org/netrants/reply-to-useful.html">'
            "`Reply-To' Munging Considered Useful</a> for a dissenting "
            'opinion. '
            'Some mailing lists have restricted '
            'posting privileges, with a parallel list devoted to discussions. '
            "Examples are `patches' or `checkin' lists, where software "
            'changes are posted by a revision control system, but discussion '
            'about the changes occurs on a developers mailing list. To '
            'support these types of mailing lists, select Explicit Reply and '
            'set the Reply-To: address option to point to the parallel list.'
        ),
    )
    posting_pipeline = forms.ChoiceField(
        label=_('Pipeline'),
        widget=forms.Select(),
        required=False,
        choices=lambda: (
            (p, p) for p in get_mailman_client().pipelines['pipelines']
        ),
        help_text=_('Type of pipeline you want to use for this mailing list'),
    )
    filter_content = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Filter content'),
        help_text=_(
            'Should Mailman filter the content of list traffic '
            'according to the settings below?\n'
            '<b>None of the settings below have any effect if this is No.</b>'
        ),
    )
    filter_types = ListOfStringsField(
        label=_('Filter types'),
        required=False,
        help_text=_(
            'MIME types to filter from the incoming posts. A list of common '
            'types can be found '
            '<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types">here </a>'  # noqa# E501
        ),
    )
    filter_extensions = ListOfStringsField(
        label=_('Filter extensions'),
        required=False,
        help_text=_('Extensions to filter from the incoming posts.'),
    )
    pass_types = ListOfStringsField(
        label=_('Pass types'),
        required=False,
        help_text=_(
            'MIME types to allow in the incoming posts. A list of common '
            'types can be found '
            '<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types">here </a>'  # noqa# E501
        ),
    )
    pass_extensions = ListOfStringsField(
        label=_('Pass extensions'),
        required=False,
        help_text=_('Extensions to allow in the incoming posts.'),
    )
    collapse_alternatives = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Collapse alternatives'),
        help_text=_(
            'Should Mailman collapse multipart/alternative to '
            'its first part content?'
        ),
    )
    filter_action = forms.ChoiceField(
        choices=FILTER_ACTION_CHOICES,
        widget=forms.RadioSelect,
        required=False,
        label=_('Filter Action'),
        help_text=FILTER_ACTION_HELP,
    )
    convert_html_to_plaintext = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Convert html to plaintext'),
        help_text=_(
            'Should Mailman convert text/html parts to plain text? '
            'This conversion happens after MIME attachments '
            'have been stripped.'
        ),
    )


class ListAutomaticResponsesForm(ListSettingsForm):
    """
    List settings for automatic responses.
    """

    autorespond_choices = (
        ('respond_and_continue', _('Respond and continue processing')),
        ('respond_and_discard', _('Respond and discard message')),
        ('none', _('No automatic response')),
    )
    autorespond_owner = forms.ChoiceField(
        choices=autorespond_choices,
        widget=forms.RadioSelect,
        label=_('Autorespond to list owner'),
        help_text=_(
            'Should Mailman send an auto-response to '
            'emails sent to the -owner address?'
        ),
    )
    autoresponse_owner_text = forms.CharField(
        label=_('Autoresponse owner text'),
        widget=forms.Textarea(),
        required=False,
        help_text=_('Auto-response text to send to -owner emails.'),
    )
    autorespond_postings = forms.ChoiceField(
        choices=autorespond_choices,
        widget=forms.RadioSelect,
        label=_('Autorespond postings'),
        help_text=_(
            'Should Mailman send an auto-response to ' 'mailing list posters?'
        ),
    )
    autoresponse_postings_text = forms.CharField(
        label=_('Autoresponse postings text'),
        widget=forms.Textarea(),
        required=False,
        help_text=_('Auto-response text to send to mailing list posters.'),
    )
    autorespond_requests = forms.ChoiceField(
        choices=autorespond_choices,
        widget=forms.RadioSelect,
        label=_('Autorespond requests'),
        help_text=_(
            'Should Mailman send an auto-response to emails sent to the '
            '-request address? If you choose yes, decide whether you want '
            'Mailman to discard the original email, or forward it on to the '
            'system as a normal mail command.'
        ),
    )
    autoresponse_request_text = forms.CharField(
        label=_('Autoresponse request text'),
        widget=forms.Textarea(),
        required=False,
        help_text=_('Auto-response text to send to -request emails.'),
    )
    autoresponse_grace_period = forms.CharField(
        label=_('Autoresponse grace period'),
        help_text=_(
            'Number of days between auto-responses to either the mailing list '
            'or -request/-owner address from the same poster. Set to zero '
            '(or negative) for no grace period (i.e. auto-respond to every '
            'message).'
        ),
    )
    respond_to_post_requests = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Notify users of held messages'),
        help_text=_(
            'Should Mailman notify users about their messages held for '
            "approval. If you say 'No', no notifications will be sent "
            'to users about the pending approval on their messages.'
        ),
    )
    send_welcome_message = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Send welcome message'),
        help_text=_(
            'Send welcome message to newly subscribed members? '
            'Turn this off only if you plan on subscribing people manually '
            "and don't want them to know that you did so. Setting this to No "
            'is most useful for transparently migrating lists from some other '
            'mailing list manager to Mailman.\n'
            'The text of Welcome message can be set via the Templates tab.'
        ),
    )
    send_goodbye_message = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        required=False,
        label=_('Send goodbye message'),
        help_text=_(
            'Send goodbye message to newly unsubscribed members? '
            'Turn this off only if you plan on unsubscribing people manually '
            "and don't want them to know that you did so.\n"
            'The text of Goodbye message can be set via the Templates tab.'
        ),
    )
    admin_immed_notify = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Admin immed notify'),
        help_text=_(
            'Should the list moderators get immediate notice of new requests, '
            'as well as daily notices about collected ones? List moderators '
            '(and list administrators) are sent daily reminders of requests '
            'pending approval, like subscriptions to a moderated list, '
            'or postings that are being held for one reason or another. '
            'Setting this option causes notices to be sent immediately on the '
            'arrival of new requests as well. '
        ),
    )
    admin_notify_mchanges = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Notify admin of membership changes'),
        help_text=_(
            'Should administrator get notices of '
            'subscribes and unsubscribes?'
        ),
    )


NEWSGROUP_MODERATION_CHOICES = (
    ('none', _('Not Moderated')),
    ('open_moderated', _('Moderated but allows for open posting')),
    ('moderated', _('Moderated')),
)


class ListIdentityForm(ListSettingsForm):
    """
    List identity settings.
    """

    advertised = forms.ChoiceField(
        choices=((True, _('Yes')), (False, _('No'))),
        widget=forms.RadioSelect,
        label=_('Show list on index page'),
        help_text=_(
            'Choose whether to include this list ' 'on the list of all lists'
        ),
    )
    description = forms.CharField(
        label=_('Short Description'),
        required=False,
        help_text=_(
            'This description is used when the mailing list is listed with '
            'other mailing lists, or in headers, and so forth. It should be '
            'as succinct as you can get it, while still identifying what the '
            'list is.'
        ),
    )
    info = forms.CharField(
        label=_('Long Description'),
        help_text=_('A longer description of this mailing list.'),
        required=False,
        widget=forms.Textarea(),
    )
    display_name = forms.CharField(
        label=_('Display name'),
        required=False,
        help_text=_('Display name is the name shown in the web interface.'),
    )
    subject_prefix = forms.CharField(
        label=_('Subject prefix'),
        strip=False,
        required=False,
    )
    preferred_language = forms.ChoiceField(
        label=_('Preferred Language'),
        required=False,
        widget=forms.Select(),
        choices=LANGUAGES,
    )
    member_roster_visibility = forms.ChoiceField(
        label=_('Members List Visibility'),
        required=False,
        widget=forms.Select(),
        choices=ROSTER_VISIBILITY_CHOICES,
        help_text=_(
            'Who is allowed to see members list for this MailingList?'
        ),
    )
    gateway_to_mail = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Gateway to mail'),
        help_text=_(
            'Flag indicating that posts to the linked newsgroup should'
            ' be gated to the list'
        ),
    )
    gateway_to_news = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('Gateway to news'),
        help_text=_(
            'Flag indicating that posts to the list should be gated to'
            ' the linked newsgroup.'
        ),
    )
    linked_newsgroup = forms.CharField(
        label=_('Linked Newsgroup'),
        required=False,
        help_text=_('The name of the linked newsgroup.'),
    )
    newsgroup_moderation = forms.ChoiceField(
        label=_('Newsgroup moderation'),
        required=False,
        widget=forms.Select(),
        choices=NEWSGROUP_MODERATION_CHOICES,
        help_text=_(
            'The moderation policy for the linked newsgroup,'
            ' if there is one.'
        ),
    )
    nntp_prefix_subject_too = forms.BooleanField(
        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
        required=False,
        label=_('NNTP Include subject prefix '),
        help_text=_(
            'Flag indicating whether the list\'s "Subject Prefix"'
            ' should be included in posts gated to usenet.'
        ),
    )

    def clean_subject_prefix(self):
        """
        Strip the leading whitespaces from the subject_prefix form field.
        """
        return self.cleaned_data.get('subject_prefix', '').lstrip()


class ListMassSubscription(forms.Form):
    """Form fields to masssubscribe users to a list."""

    emails = ListOfStringsField(
        label=_('Emails to mass subscribe'),
        help_text=_(
            'The following formats are accepted:\n'
            'jdoe@example.com\n'
            '&lt;jdoe@example.com&gt;\n'
            'John Doe &lt;jdoe@example.com&gt;\n'
            '"John Doe" &lt;jdoe@example.com&gt;\n'
            'jdoe@example.com (John Doe)\n'
            'Use the last three to associate a display name with the address\n'
        ),
    )

    pre_confirmed = forms.BooleanField(
        label=_('Pre confirm'),
        initial=True,
        required=False,
        help_text=_(
            'If checked, users will not have to confirm their subscription.'
        ),
        widget=forms.CheckboxInput(),
    )

    pre_approved = forms.BooleanField(
        label=_('Pre approved'),
        initial=True,
        required=False,
        help_text=_(
            'If checked, moderators will not have to approve the subscription'
            ' request.',
        ),
        widget=forms.CheckboxInput(),
    )

    pre_verified = forms.BooleanField(
        label=_('Pre Verified'),
        initial=False,
        required=False,
        help_text=_(
            'If checked, users will not have to verify that their '
            'email address is valid.'
        ),
        widget=forms.CheckboxInput(),
    )

    invitation = forms.BooleanField(
        label=_('Invitation'),
        initial=False,
        required=False,
        help_text=_(
            'If checked, the other checkboxes are ignored and the users will '
            'be sent an invitation to join the list and will be subscribed '
            'upon acceptance thereof.'
        ),
        widget=forms.CheckboxInput(),
    )

    send_welcome_message = forms.ChoiceField(
        choices=(
            (True, _('Yes')),
            (False, _('No')),
            ('default', _('List default')),
        ),
        widget=forms.RadioSelect,
        initial='default',
        required=False,
        label=_('Send welcome message'),
        help_text=_(
            'If set to "Yes" or "No", List\'s default setting of '
            'send_welcome_message will be ignored for these subscribers and a'
            ' welcome message will be sent or not sent based on the choice.'
        ),
    )

    def clean_send_welcome_message(self):
        """Choose from True or False. Any other value is equivalent to None."""
        data = self.cleaned_data['send_welcome_message']
        if data in ('True', 'False'):
            return data
        # None implies this value is unset and isn't passed on to Core in API
        # call.
        return None


class ListMassRemoval(forms.Form):

    """Form fields to remove multiple list users."""

    emails = ListOfStringsField(
        label=_('Emails to Unsubscribe'),
        help_text=_('Add one email address on each line'),
    )

    class Meta:

        """
        Class to define the name of the fieldsets and what should be
        included in each.
        """

        layout = [['Mass Removal', 'emails']]


class ListHeaderMatchForm(forms.Form):
    """Edit a list's header match."""

    HM_ACTION_CHOICES = [(None, _('Default antispam action'))] + [
        a for a in ACTION_CHOICES if a[0] != 'defer'
    ]

    header = forms.CharField(
        label=_('Header'),
        help_text=_('Email header to filter on (case-insensitive).'),
        error_messages={
            'required': _('Please enter a header.'),
            'invalid': _('Please enter a valid header.'),
        },
    )
    pattern = forms.CharField(
        label=_('Pattern'),
        help_text=_(
            "Regular expression matching the header's value "
            '(case-insensitive).'
        ),
        error_messages={
            'required': _('Please enter a pattern.'),
            'invalid': _('Please enter a valid pattern.'),
        },
    )
    action = forms.ChoiceField(
        label=_('Action'),
        error_messages={'invalid': _('Please enter a valid action.')},
        required=False,
        choices=HM_ACTION_CHOICES,
        help_text=_('Action to take when a header matches'),
    )


class ListHeaderMatchFormset(forms.BaseFormSet):
    def clean(self):
        """Checks that no two header matches have the same order."""
        if any(self.errors):
            # Don't bother validating the formset unless
            # each form is valid on its own
            return
        orders = []
        for form in self.forms:
            try:
                order = form.cleaned_data['ORDER']
            except KeyError:
                continue
            if order in orders:
                raise forms.ValidationError(
                    'Header matches must have' ' distinct orders.'
                )
            orders.append(order)


class MemberModeration(forms.Form):
    """
    Form handling the member's moderation_action.
    """

    moderation_action = moderation_action_field()


class ChangeSubscriptionForm(forms.Form):

    subscriber = forms.ChoiceField(
        label=_('Select Email'),
        required=False,
        widget=forms.Select(),
        validators=[
            validate_uuid_or_email,
        ],
    )

    member_id = forms.CharField(
        required=True,
        label='',
        widget=forms.TextInput(attrs={'readonly': True, 'hidden': True}),
    )

    def __init__(self, user_emails, user_id, primary_email, *args, **kwargs):
        super(ChangeSubscriptionForm, self).__init__(*args, **kwargs)
        choices = list((address, address) for address in user_emails)
        if primary_email and user_id:
            choices.insert(
                0, (user_id, _('Primary Address ({})').format(primary_email))
            )
        self.fields['subscriber'].choices = choices


class TemplateUpdateForm(forms.ModelForm):
    data = forms.CharField(
        label=_('Data'),
        required=False,
        strip=False,
        widget=forms.Textarea(),
        help_text=_email_template_help_text,
    )

    class Meta:
        model = EmailTemplate
        fields = ['data']


class TokenConfirmForm(forms.Form):
    """Form to confirm pending (un)subscription requests from User."""

    token = forms.CharField(
        required=True,
        label='',
        widget=forms.TextInput(attrs={'readonly': True, 'hidden': True}),
    )



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#


from django import forms


try:
    from django.utils.encoding import smart_str
except ImportError:
    # django < 4.0
    from django.utils.encoding import smart_text as smart_str

from django.utils.translation import gettext_lazy as _

from postorius.utils import with_empty_choice


DELIVERY_MODE_CHOICES = (
    ('regular', _('Regular')),
    ('plaintext_digests', _('Plain Text Digests')),
    ('mime_digests', _('MIME Digests')),
    ('summary_digests', _('Summary Digests')),
)


DELIVERY_STATUS_CHOICES = (
    ('enabled', _('Enabled')),
    ('by_user', _('Disabled')),
    ('by_moderator', _('Disabled by Owner')),
    ('by_bounces', _('Disabled by Bounces')),
)

ACTION_CHOICES = (
    ('hold', _('Hold for moderation')),
    ('reject', _('Reject (with notification)')),
    ('discard', _('Discard (no notification)')),
    ('accept', _('Accept immediately (bypass other rules)')),
    ('defer', _('Default processing')),
)


class ListOfStringsField(forms.Field):
    widget = forms.widgets.Textarea

    def prepare_value(self, value):
        if isinstance(value, list):
            value = '\n'.join(value)
        return value

    def to_python(self, value):
        """Returns a list of Unicode object."""
        if value in self.empty_values:
            return []
        result = []
        for line in value.splitlines():
            line = line.strip()
            if not line:
                continue
            result.append(smart_str(line))
        return result


class NullBooleanRadioSelect(forms.RadioSelect):
    """
    This is necessary to detect that such a field has not been changed.
    """

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        return {
            '2': True,
            True: True,
            'True': True,
            '3': False,
            'False': False,
            False: False,
        }.get(value, None)


class SelectWidget(forms.Select):
    """
    Subclass of Django's select widget that allows disabling options.
    """

    def __init__(self, *args, **kwargs):
        self._disabled_choices = []
        super().__init__(*args, **kwargs)

    @property
    def disabled_choices(self):
        return self._disabled_choices

    @disabled_choices.setter
    def disabled_choices(self, other):
        self._disabled_choices = other

    def create_option(self, name, value, *args, **kwargs):
        option_dict = super().create_option(name, value, *args, **kwargs)
        if value in self.disabled_choices:
            option_dict['attrs']['disabled'] = 'disabled'
            return option_dict
        return option_dict


class SiteModelChoiceField(forms.ModelChoiceField):
    def label_from_instance(self, obj):
        return '%s (%s)' % (obj.name, obj.domain)


class MultipleChoiceForm(forms.Form):
    class MultipleChoiceField(forms.MultipleChoiceField):
        def validate(self, value):
            pass

    choices = MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple,
    )

    def clean_choices(self):
        if len(self.cleaned_data['choices']) < 1:
            raise forms.ValidationError(_('Make at least one selection'))
        return self.cleaned_data['choices']


def delivery_mode_field(default=None):

    return forms.ChoiceField(
        widget=forms.Select(),
        choices=with_empty_choice(DELIVERY_MODE_CHOICES),
        required=False,
        initial=default,
        label=_('Delivery mode'),
        help_text=_(
            "If you select digests , you'll get posts bundled "
            'together (usually one per day but possibly more on busy lists), '
            "instead of singly when they're sent. Your mail reader may or "
            'may not support MIME digests. In general MIME digests are '
            'preferred, but if you have a problem reading them, select '
            'plain text digests. '
            'Summary Digests are currently equivalent to MIME Digests.'
        ),
    )


def delivery_status_field(choices=None, widget=None):
    if not choices:
        choices = with_empty_choice(DELIVERY_STATUS_CHOICES)

    if not widget:
        widget = SelectWidget

    return forms.ChoiceField(
        widget=widget(),
        choices=choices,
        required=False,
        label=_('Delivery status'),
        help_text=_(
            'Set this option to Enabled to receive messages posted to this '
            'mailing list. Set it to Disabled if you want to stay subscribed, '
            "but don't want mail delivered to you for a while (e.g. you're "
            "going on vacation). If you disable mail delivery, don't forget "
            'to re-enable it when you come back; it will not be automatically '
            're-enabled.'
        ),
    )


def moderation_action_field():
    return forms.ChoiceField(
        widget=forms.Select(),
        label=_('Moderation'),
        required=False,
        choices=[(None, _('List default'))] + list(ACTION_CHOICES),
        help_text=_(
            'Default action to take when this member posts to the list. \n'
            "List default -- follow the list's default member action. \n"
            'Hold -- This holds the message for approval by the list '
            'moderators. \n'
            'Reject -- this automatically rejects the message by sending a '
            "bounce notice to the post's author. The text of the bounce "
            'notice can be configured by you. \n'
            'Discard -- this simply discards the message, with no notice '
            "sent to the post's author. \n"
            'Accept -- accepts any postings without any further checks. \n'
            'Default Processing -- run additional checks and accept '
            'the message. \n'
        ),
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-13 09:48


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0002_auto_20160210_0721'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='addressconfirmationprofile',
            name='user',
        ),
        migrations.DeleteModel(
            name='AddressConfirmationProfile',
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

import base64
import os

from django.core.management.base import BaseCommand

from django_mailman3.lib.mailman import get_mailman_client


PASSWORD_BYTES = 32


class Command(BaseCommand):

    help = """Reset passwords of all users in Mailman Core. This does not
    affect the login passwords of users in Django or any other social
    authentication. Users will be able to login with their current passwords.
    Mailman Core maintains a second set of passwords for every user, which
    would be set to a random value of base64 encode 32 bytes."""

    def handle(self, *args, **kwargs):
        client = get_mailman_client()
        for user in self._get_all_users(client):
            self._reset_password(user)

    def _get_all_users(self, client):
        """Given a mailmanclient.Client instance, returns an iterator of
        paginated user records.
        """
        page = client.get_user_page(count=50, page=1)
        while True:
            for user in page:
                yield user
            if page.has_next:
                page = page.next
            else:
                break

    def _reset_password(self, user):
        """Given a mailmanclient.restobject.user.User object, reset its
        password to None in the database.
        """
        user.password = self._get_random_password()
        user.save()
        msg = 'Password reset for {}'.format(user)
        self.stdout.write(self.style.SUCCESS(msg))

    def _get_random_password(self):
        """Generate a random password for a user."""
        tok = os.urandom(PASSWORD_BYTES)
        return base64.urlsafe_b64encode(tok).rstrip(b'=').decode('ascii')



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

import logging
from enum import Enum
from urllib.error import HTTPError
from urllib.parse import urljoin

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from django.http import Http404
from django.urls import reverse
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

from mailmanclient import MailmanConnectionError

from postorius.template_list import TEMPLATES_LIST
from postorius.utils import LANGUAGES, get_mailman_client


logger = logging.getLogger(__name__)

_email_template_help_text = _(
    'Note: Do not add any secret content in templates as they are '
    'publicly accessible.\n'
    'You can use these variables in the templates. \n'
    '$hyperkitty_url: Permalink to archived message in Hyperkitty\n'
    '$listname: Name of the Mailing List e.g. ant@example.com \n'
    '$list_id: The List-ID header e.g. ant.example.com \n'
    '$display_name: Display name of the mailing list e.g. Ant \n'
    '$short_listname: Local part of the listname e.g. ant \n'
    '$domain: The domain part of the listname e.g. example.com \n'
    "$info: The mailing list's longer descriptive text \n"
    '$request_email: The email address for -request address \n'
    '$owner_email: The email address for -owner address \n'
    '$site_email: The email address to reach the owners of the site \n'
    "$language: The two letter language code for list's preferred language e.g. fr, en, de \n"  # noqa: E501
)


class SubscriptionMode(Enum):
    """Valid values for Member.subscription_mode"""

    as_address = 'as_address'
    as_user = 'as_user'


class MemberRole(Enum):
    """List of roles a Member resource can have.

    The primary purpose of this enum right now is only to add the
    role values to translations. At some point, we want to use MemberRole.owner
    in all the places we are using the literal role as string.
    """

    owner = _('owner')
    member = _('member')
    nonmember = _('nonmember')
    moderator = _('moderator')


class MailmanApiError(Exception):
    """Raised if the API is not available."""

    pass


class Mailman404Error(Exception):
    """Proxy exception. Raised if the API returns 404."""

    pass


class MailmanRestManager(object):
    """Manager class to give a model class CRUD access to the API.
    Returns objects (or lists of objects) retrieved from the API.
    """

    def __init__(self, resource_name, resource_name_plural, cls_name=None):
        self.resource_name = resource_name
        self.resource_name_plural = resource_name_plural

    def all(self):
        try:
            return getattr(get_mailman_client(), self.resource_name_plural)
        except AttributeError:
            raise MailmanApiError
        except MailmanConnectionError as e:
            raise MailmanApiError(e)

    def get(self, *args, **kwargs):
        try:
            method = getattr(get_mailman_client(), 'get_' + self.resource_name)
            return method(*args, **kwargs)
        except AttributeError as e:
            raise MailmanApiError(e)
        except HTTPError as e:
            if e.code == 404:
                raise Mailman404Error('Mailman resource could not be found.')
            else:
                raise
        except MailmanConnectionError as e:
            raise MailmanApiError(e)

    def get_or_404(self, *args, **kwargs):
        """Similar to `self.get` but raises standard Django 404 error."""
        try:
            return self.get(*args, **kwargs)
        except Mailman404Error:
            raise Http404
        except MailmanConnectionError as e:
            raise MailmanApiError(e)

    def create(self, *args, **kwargs):
        try:
            method = getattr(
                get_mailman_client(), 'create_' + self.resource_name
            )
            return method(*args, **kwargs)
        except AttributeError as e:
            raise MailmanApiError(e)
        except HTTPError as e:
            if e.code == 409:
                raise MailmanApiError
            else:
                raise
        except MailmanConnectionError:
            raise MailmanApiError

    def delete(self):
        """Not implemented since the objects returned from the API
        have a `delete` method of their own.
        """
        pass


class MailmanListManager(MailmanRestManager):
    def __init__(self):
        super(MailmanListManager, self).__init__('list', 'lists')

    def all(self, advertised=False):
        try:
            method = getattr(
                get_mailman_client(), 'get_' + self.resource_name_plural
            )
            return method(advertised=advertised)
        except AttributeError:
            raise MailmanApiError
        except MailmanConnectionError as e:
            raise MailmanApiError(e)

    def by_mail_host(self, mail_host, advertised=False):
        objects = self.all(advertised)
        host_objects = []
        for obj in objects:
            if obj.mail_host == mail_host:
                host_objects.append(obj)
        return host_objects


class MailmanUserManager(MailmanRestManager):
    def __init__(self):
        super(MailmanUserManager, self).__init__('user', 'users')


class MailmanRestModel(object):
    """Simple REST Model class to make REST API calls Django style."""

    MailmanApiError = MailmanApiError
    DoesNotExist = Mailman404Error

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def save(self):
        """Proxy function for `objects.create`.
        (REST API uses `create`, while Django uses `save`.)
        """
        self.objects.create(*self.args, **self.kwargs)


class Domain(MailmanRestModel):
    """Domain model class."""

    objects = MailmanRestManager('domain', 'domains')


class List(MailmanRestModel):
    """List model class."""

    objects = MailmanListManager()


class MailmanUser(MailmanRestModel):
    """MailmanUser model class."""

    objects = MailmanUserManager()


class Member(MailmanRestModel):
    """Member model class."""

    objects = MailmanRestManager('member', 'members')


class Style(MailmanRestModel):
    """ """

    objects = MailmanRestManager(None, 'styles')


TEMPLATE_CONTEXT_CHOICES = (
    ('site', 'Site Wide'),
    ('domain', 'Domain Wide'),
    ('list', 'MailingList Wide'),
)


class EmailTemplate(models.Model):
    """A Template represents contents of partial or complete emails sent out by
    Mailman Core on various events or when an action is required. Headers and
    Footers on emails for decorations are also repsented as templates.
    """

    # Ease differentiating the various Mailman templates by providing the
    # template file's name (key) prepended in square brackets to the
    # template's purpose (value).
    _templates_list_choices = [
        (t[0], format_lazy('[{key}] - {value}', key=t[0], value=t[1]))
        for t in TEMPLATES_LIST
    ]

    name = models.CharField(
        max_length=100,
        choices=_templates_list_choices,
        help_text=_('Choose the template you want to customize.'),
    )
    data = models.TextField(
        help_text=_email_template_help_text,
        blank=True,
    )
    language = models.CharField(
        max_length=5,
        choices=LANGUAGES,
        help_text=_(
            "Language for the template, this should be the list's preferred language."  # noqa: E501
        ),
        blank=True,
    )
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)
    context = models.CharField(max_length=50, choices=TEMPLATE_CONTEXT_CHOICES)
    identifier = models.CharField(blank=True, max_length=100)

    class Meta:
        unique_together = ('name', 'identifier', 'language')

    def __str__(self):
        return '<EmailTemplate {0} for {1}>'.format(self.name, self.context)

    @property
    def description(self):
        """Return the long description of template that is human readable."""
        return dict(TEMPLATES_LIST)[self.name]

    @property
    def api_url(self):
        """API url is the remote url that Core can use to fetch templates"""
        base_url = getattr(settings, 'POSTORIUS_TEMPLATE_BASE_URL', None)
        if not base_url:
            raise ImproperlyConfigured(
                'Setting "POSTORIUS_TEMPLATE_BASE_URL" is not configured.'
            )
        resource_url = reverse(
            'rest_template',
            kwargs=dict(
                context=self.context,
                identifier=self.identifier,
                name=self.name,
            ),
        )
        return urljoin(base_url, resource_url)

    def _get_context_obj(self):
        if self.context == 'list':
            obj = List.objects.get_or_404(fqdn_listname=self.identifier)
        elif self.context == 'domain':
            obj = Domain.objects.get_or_404(mail_host=self.identifier)
        elif self.context == 'site':
            obj = get_mailman_client()
        else:
            obj = None
        return obj

    def _update_core(self, deleted=False):
        obj = self._get_context_obj()
        if obj is None:
            return

        if deleted:
            # POST'ing an empty string will delete this record in Core.
            api_url = ''
        else:
            # Use the API endpoint of self that Core can use to fetch this.
            api_url = self.api_url
        obj.set_template(self.name, api_url)


@receiver(post_save, sender=EmailTemplate)
def update_core_post_update(sender, **kwargs):
    kwargs['instance']._update_core()


@receiver(post_delete, sender=EmailTemplate)
def update_core_post_delete(sender, **kwargs):
    kwargs['instance']._update_core(deleted=True)
